@@ -48,6 +48,8 @@
 GST_DEBUG_CATEGORY_STATIC (celtdec_debug);
 #define GST_CAT_DEFAULT celtdec_debug
 
+#define DEC_MAX_FRAME_SIZE 2000
+
 static GstStaticPadTemplate celt_dec_src_factory =
 GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
@@ -495,14 +497,24 @@ celt_dec_chain_parse_header (GstCeltDec * dec, GstBuffer * buf)
   if (memcmp (dec->header.codec_id, "CELT    ", 8) != 0)
     goto invalid_header;
 
+#ifdef HAVE_CELT_0_7
+  dec->mode =
+      celt_mode_create (dec->header.sample_rate,
+      dec->header.frame_size, &error);
+#else
   dec->mode =
       celt_mode_create (dec->header.sample_rate, dec->header.nb_channels,
       dec->header.frame_size, &error);
+#endif
   if (!dec->mode)
     goto mode_init_failed;
 
   /* initialize the decoder */
+#ifdef HAVE_CELT_0_7
+  dec->state = celt_decoder_create (dec->mode, dec->header.nb_channels, &error);
+#else
   dec->state = celt_decoder_create (dec->mode);
+#endif
   if (!dec->state)
     goto init_failed;
 
@@ -519,6 +531,9 @@ celt_dec_chain_parse_header (GstCeltDec * dec, GstBuffer * buf)
       "endianness", G_TYPE_INT, G_BYTE_ORDER,
       "width", G_TYPE_INT, 16, "depth", G_TYPE_INT, 16, NULL);
 
+  GST_DEBUG_OBJECT (dec, "rate=%d channels=%d frame-size=%d",
+      dec->header.sample_rate, dec->header.nb_channels, dec->frame_size);
+
   if (!gst_pad_set_caps (dec->srcpad, caps))
     goto nego_failed;
 
@@ -540,8 +555,13 @@ mode_init_failed:
   }
 init_failed:
   {
+#ifdef HAVE_CELT_0_7
+    GST_ELEMENT_ERROR (GST_ELEMENT (dec), STREAM, DECODE,
+        (NULL), ("couldn't initialize decoder: %d", error));
+#else
     GST_ELEMENT_ERROR (GST_ELEMENT (dec), STREAM, DECODE,
         (NULL), ("couldn't initialize decoder"));
+#endif
     return GST_FLOW_ERROR;
   }
 nego_failed:
@@ -607,6 +627,7 @@ celt_dec_chain_parse_data (GstCeltDec * dec, GstBuffer * buf,
   GstBuffer *outbuf;
   gint16 *out_data;
   gint error = CELT_OK;
+  int skip = 0;
 
   if (timestamp != -1) {
     dec->segment.last_stop = timestamp;
@@ -634,6 +655,10 @@ celt_dec_chain_parse_data (GstCeltDec * dec, GstBuffer * buf,
     size = 0;
   }
 
+  if (dec->discont) {
+    celt_mode_info (dec->mode, CELT_GET_LOOKAHEAD, &skip);
+  }
+
   res = gst_pad_alloc_buffer_and_set_caps (dec->srcpad,
       GST_BUFFER_OFFSET_NONE, dec->frame_size * dec->header.nb_channels * 2,
       GST_PAD_CAPS (dec->srcpad), &outbuf);
@@ -653,6 +678,14 @@ celt_dec_chain_parse_data (GstCeltDec * dec, GstBuffer * buf,
     return GST_FLOW_ERROR;
   }
 
+  if (skip > 0) {
+    GST_ERROR ("skipping %d samples", skip);
+    GST_BUFFER_DATA (outbuf) = GST_BUFFER_DATA (outbuf) +
+        skip * dec->header.nb_channels * 2;
+    GST_BUFFER_SIZE (outbuf) = GST_BUFFER_SIZE (outbuf) -
+        skip * dec->header.nb_channels * 2;
+  }
+
   if (dec->granulepos == -1) {
     if (dec->segment.format != GST_FORMAT_TIME) {
       GST_WARNING_OBJECT (dec, "segment not initialized or not TIME format");
@@ -664,12 +697,21 @@ celt_dec_chain_parse_data (GstCeltDec * dec, GstBuffer * buf,
     GST_DEBUG_OBJECT (dec, "granulepos=%" G_GINT64_FORMAT, dec->granulepos);
   }
 
+  if (!GST_CLOCK_TIME_IS_VALID (timestamp))
+    timestamp = gst_util_uint64_scale_int (dec->granulepos - dec->frame_size,
+        GST_SECOND, dec->header.sample_rate);
+
+  GST_DEBUG_OBJECT (dec, "timestamp=%" GST_TIME_FORMAT,
+      GST_TIME_ARGS (timestamp));
+
   GST_BUFFER_OFFSET (outbuf) = dec->granulepos - dec->frame_size;
   GST_BUFFER_OFFSET_END (outbuf) = dec->granulepos;
-  GST_BUFFER_TIMESTAMP (outbuf) =
-      gst_util_uint64_scale_int (dec->granulepos - dec->frame_size, GST_SECOND,
-      dec->header.sample_rate);
+  GST_BUFFER_TIMESTAMP (outbuf) = timestamp;
   GST_BUFFER_DURATION (outbuf) = dec->frame_duration;
+  if (dec->discont) {
+    GST_BUFFER_FLAG_SET (outbuf, GST_BUFFER_FLAG_DISCONT);
+    dec->discont = 0;
+  }
 
   dec->granulepos += dec->frame_size;
   dec->segment.last_stop += dec->frame_duration;
@@ -694,6 +736,10 @@ celt_dec_chain (GstPad * pad, GstBuffer * buf)
 
   dec = GST_CELT_DEC (gst_pad_get_parent (pad));
 
+  if (GST_BUFFER_IS_DISCONT (buf)) {
+    dec->discont = TRUE;
+  }
+
   if (dec->packetno == 0)
     res = celt_dec_chain_parse_header (dec, buf);
   else if (dec->packetno == 1)
