@@ -72,12 +72,6 @@ static GstStaticPadTemplate src_factory = GST_STATIC_PAD_TEMPLATE ("src",
         "channels = (int) [ 1, 2 ], " "frame-size = (int) [ 64, 512 ]")
     );
 
-static const GstElementDetails celtenc_details =
-GST_ELEMENT_DETAILS ("Celt audio encoder",
-    "Codec/Encoder/Audio",
-    "Encodes audio in Celt format",
-    "Sebastian Dröge <sebastian.droege@collabora.co.uk>");
-
 #define DEFAULT_BITRATE         64
 #define DEFAULT_FRAMESIZE       480
 #define DEFAULT_CBR             TRUE
@@ -623,7 +617,9 @@ gst_celt_enc_setup (GstCeltEnc * enc)
     celt_encoder_ctl (enc->state, CELT_SET_VBR_RATE (enc->bitrate), 0);
   }
 #endif
+#ifdef CELT_SET_COMPLEXITY
   celt_encoder_ctl (enc->state, CELT_SET_COMPLEXITY (enc->complexity), 0);
+#endif
 
   GST_LOG_OBJECT (enc, "we have frame size %d", enc->frame_size);
 
