@@ -20,9 +20,9 @@
 
 /**
  * SECTION:element-faac
- * @seealso: faad
+ * @see_also: faad
  *
- * faac encodes raw audio to AAC (MPEG-4 part 10) streams.
+ * faac encodes raw audio to AAC (MPEG-4 part 3) streams.
  *
  * The #GstFaac:outputformat property determines whether or not the
  * AAC data needs additional framing provided by a container
@@ -36,9 +36,9 @@
  *
  * <refsect2>
  * <title>Example launch line</title>
- * <programlisting>
+ * |[
  * gst-launch audiotestsrc wave=sine num-buffers=100 ! audioconvert ! faac ! matroskamux ! filesink location=sine.mkv
- * </programlisting>
+ * ]| Encode a sine beep as aac and write to matroska container.
  * </refsect2>
  */
 
@@ -48,6 +48,8 @@
 #include <stdlib.h>
 #include <string.h>
 
+#include <gst/audio/multichannel.h>
+
 #include "gstfaac.h"
 
 #define SINK_CAPS \
@@ -77,7 +79,8 @@
     "audio/mpeg, "                     \
     "mpegversion = (int) { 4, 2 }, "   \
     "channels = (int) [ 1, 6 ], "      \
-    "rate = (int) [ 8000, 96000 ]"
+    "rate = (int) [ 8000, 96000 ], "   \
+    "stream-format = (string) { adts, raw } "
 static GstStaticPadTemplate src_template = GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
     GST_PAD_ALWAYS,
@@ -88,12 +91,6 @@ static GstStaticPadTemplate sink_template = GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_ALWAYS,
     GST_STATIC_CAPS (SINK_CAPS));
 
-static const GstElementDetails gst_faac_details =
-GST_ELEMENT_DETAILS ("AAC audio encoder",
-    "Codec/Encoder/Audio",
-    "Free MPEG-2/4 AAC encoder",
-    "Ronald Bultje <rbultje@ronald.bitfreak.net>");
-
 enum
 {
   ARG_0,
@@ -119,6 +116,7 @@ static void gst_faac_get_property (GObject * object,
 static gboolean gst_faac_sink_event (GstPad * pad, GstEvent * event);
 static gboolean gst_faac_configure_source_pad (GstFaac * faac);
 static gboolean gst_faac_sink_setcaps (GstPad * pad, GstCaps * caps);
+static GstCaps *gst_faac_sink_getcaps (GstPad * pad);
 static GstFlowReturn gst_faac_push_buffers (GstFaac * faac, gboolean force);
 static GstFlowReturn gst_faac_chain (GstPad * pad, GstBuffer * data);
 static GstStateChangeReturn gst_faac_change_state (GstElement * element,
@@ -180,7 +178,10 @@ gst_faac_base_init (GstFaacClass * klass)
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&sink_template));
 
-  gst_element_class_set_details (element_class, &gst_faac_details);
+  gst_element_class_set_details_simple (element_class, "AAC audio encoder",
+      "Codec/Encoder/Audio",
+      "Free MPEG-2/4 AAC encoder",
+      "Ronald Bultje <rbultje@ronald.bitfreak.net>");
 
   GST_DEBUG_CATEGORY_INIT (faac_debug, "faac", 0, "AAC encoding");
 }
@@ -295,6 +296,8 @@ gst_faac_init (GstFaac * faac)
       GST_DEBUG_FUNCPTR (gst_faac_chain));
   gst_pad_set_setcaps_function (faac->sinkpad,
       GST_DEBUG_FUNCPTR (gst_faac_sink_setcaps));
+  gst_pad_set_getcaps_function (faac->sinkpad,
+      GST_DEBUG_FUNCPTR (gst_faac_sink_getcaps));
   gst_pad_set_event_function (faac->sinkpad,
       GST_DEBUG_FUNCPTR (gst_faac_sink_event));
   gst_element_add_pad (GST_ELEMENT (faac), faac->sinkpad);
@@ -346,6 +349,82 @@ gst_faac_close_encoder (GstFaac * faac)
   faac->offset = 0;
 }
 
+static const GstAudioChannelPosition aac_channel_positions[][8] = {
+  {GST_AUDIO_CHANNEL_POSITION_FRONT_MONO},
+  {GST_AUDIO_CHANNEL_POSITION_FRONT_LEFT,
+      GST_AUDIO_CHANNEL_POSITION_FRONT_RIGHT},
+  {
+        GST_AUDIO_CHANNEL_POSITION_FRONT_CENTER,
+        GST_AUDIO_CHANNEL_POSITION_FRONT_LEFT,
+        GST_AUDIO_CHANNEL_POSITION_FRONT_RIGHT,
+      },
+  {
+        GST_AUDIO_CHANNEL_POSITION_FRONT_CENTER,
+        GST_AUDIO_CHANNEL_POSITION_FRONT_LEFT,
+        GST_AUDIO_CHANNEL_POSITION_FRONT_RIGHT,
+      GST_AUDIO_CHANNEL_POSITION_REAR_CENTER},
+  {
+        GST_AUDIO_CHANNEL_POSITION_FRONT_CENTER,
+        GST_AUDIO_CHANNEL_POSITION_FRONT_LEFT,
+        GST_AUDIO_CHANNEL_POSITION_FRONT_RIGHT,
+        GST_AUDIO_CHANNEL_POSITION_REAR_LEFT,
+      GST_AUDIO_CHANNEL_POSITION_REAR_RIGHT},
+  {
+        GST_AUDIO_CHANNEL_POSITION_FRONT_CENTER,
+        GST_AUDIO_CHANNEL_POSITION_FRONT_LEFT,
+        GST_AUDIO_CHANNEL_POSITION_FRONT_RIGHT,
+        GST_AUDIO_CHANNEL_POSITION_REAR_LEFT,
+        GST_AUDIO_CHANNEL_POSITION_REAR_RIGHT,
+      GST_AUDIO_CHANNEL_POSITION_LFE}
+};
+
+static GstCaps *
+gst_faac_sink_getcaps (GstPad * pad)
+{
+  static volatile gsize sinkcaps = 0;
+
+  if (g_once_init_enter (&sinkcaps)) {
+    GstCaps *tmp = gst_caps_new_empty ();
+    GstStructure *s, *t;
+    gint i, c;
+
+    s = gst_structure_new ("audio/x-raw-int",
+        "endianness", G_TYPE_INT, G_BYTE_ORDER,
+        "signed", G_TYPE_BOOLEAN, TRUE,
+        "width", G_TYPE_INT, 16,
+        "depth", G_TYPE_INT, 16, "rate", GST_TYPE_INT_RANGE, 8000, 96000, NULL);
+
+    for (i = 1; i <= 6; i++) {
+      GValue chanpos = { 0 };
+      GValue pos = { 0 };
+
+      t = gst_structure_copy (s);
+
+      gst_structure_set (t, "channels", G_TYPE_INT, i, NULL);
+
+      g_value_init (&chanpos, GST_TYPE_ARRAY);
+      g_value_init (&pos, GST_TYPE_AUDIO_CHANNEL_POSITION);
+
+      for (c = 0; c < i; c++) {
+        g_value_set_enum (&pos, aac_channel_positions[i - 1][c]);
+        gst_value_array_append_value (&chanpos, &pos);
+      }
+      g_value_unset (&pos);
+
+      gst_structure_set_value (t, "channel-positions", &chanpos);
+      g_value_unset (&chanpos);
+      gst_caps_append_structure (tmp, t);
+    }
+    gst_structure_free (s);
+
+    GST_DEBUG_OBJECT (pad, "Generated sinkcaps: %" GST_PTR_FORMAT, tmp);
+
+    g_once_init_leave (&sinkcaps, (gsize) tmp);
+  }
+
+  return gst_caps_ref ((GstCaps *) sinkcaps);
+}
+
 static gboolean
 gst_faac_sink_setcaps (GstPad * pad, GstCaps * caps)
 {
@@ -490,16 +569,28 @@ gst_faac_configure_source_pad (GstFaac * faac)
     conf->bitRate = maxbitrate;
   }
 
+  /* default 0 to start with, libfaac chooses based on bitrate */
+  conf->bandWidth = 0;
+
   if (!faacEncSetConfiguration (faac->handle, conf))
     goto set_failed;
 
+  /* let's see what really happened,
+   * note that this may not really match desired rate */
+  GST_DEBUG_OBJECT (faac, "average bitrate: %lu kbps",
+      (conf->bitRate + 500) / 1000 * faac->channels);
+  GST_DEBUG_OBJECT (faac, "quantization quality: %ld", conf->quantqual);
+  GST_DEBUG_OBJECT (faac, "bandwidth: %d Hz", conf->bandWidth);
+
   /* now create a caps for it all */
   srccaps = gst_caps_new_simple ("audio/mpeg",
       "mpegversion", G_TYPE_INT, mpegversion,
       "channels", G_TYPE_INT, faac->channels,
-      "rate", G_TYPE_INT, faac->samplerate, NULL);
+      "rate", G_TYPE_INT, faac->samplerate,
+      "stream-format", G_TYPE_STRING, (faac->outputformat ? "adts" : "raw"),
+      NULL);
 
-  if (mpegversion == 4) {
+  if (!faac->outputformat) {
     GstBuffer *codec_data;
     guint8 *config = NULL;
     gulong config_len = 0;
