@@ -98,12 +98,7 @@ static GstStateChangeReturn rsn_dvdbin_change_state (GstElement * element,
 static void
 rsn_dvdbin_base_init (gpointer gclass)
 {
-  static GstElementDetails element_details = {
-    "rsndvdbin",
-    "Generic/Bin/Player",
-    "DVD playback element",
-    "Jan Schmidt <thaytan@noraisin.net>"
-  };
+
   GstElementClass *element_class = GST_ELEMENT_CLASS (gclass);
 
   gst_element_class_add_pad_template (element_class,
@@ -112,7 +107,9 @@ rsn_dvdbin_base_init (gpointer gclass)
       gst_static_pad_template_get (&audio_src_template));
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&subpicture_src_template));
-  gst_element_class_set_details (element_class, &element_details);
+  gst_element_class_set_details_simple (element_class, "rsndvdbin",
+      "Generic/Bin/Player",
+      "DVD playback element", "Jan Schmidt <thaytan@noraisin.net>");
 
   element_class->change_state = GST_DEBUG_FUNCPTR (rsn_dvdbin_change_state);
 }
@@ -178,7 +175,7 @@ rsn_dvdbin_uri_get_type (void)
 static gchar **
 rsn_dvdbin_uri_get_protocols (void)
 {
-  static gchar *protocols[] = { "dvd", NULL };
+  static gchar *protocols[] = { (char *) "dvd", NULL };
 
   return protocols;
 }
