@@ -1,5 +1,5 @@
 /* GStreamer
- * Copyright (C) 2008 Jan Schmidt <thaytan@noraisin.net>
+ * Copyright (C) 2008-2009 Jan Schmidt <thaytan@noraisin.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -27,6 +27,7 @@
 #include <gst/gst.h>
 #include <gst/gst-i18n-plugin.h>
 #include <gst/interfaces/navigation.h>
+#include <gst/video/video.h>
 
 #include "resindvdsrc.h"
 
@@ -97,8 +98,8 @@ static GstFormat chapter_format;
 
 static void rsn_dvdsrc_register_extra (GType rsn_dvdsrc_type);
 
-GST_BOILERPLATE_FULL (resinDvdSrc, rsn_dvdsrc, RsnBaseSrc,
-    RSN_TYPE_BASE_SRC, rsn_dvdsrc_register_extra);
+GST_BOILERPLATE_FULL (resinDvdSrc, rsn_dvdsrc, GstBaseSrc,
+    GST_TYPE_BASE_SRC, rsn_dvdsrc_register_extra);
 
 static gboolean read_vts_info (resinDvdSrc * src);
 
@@ -109,15 +110,15 @@ static void rsn_dvdsrc_get_property (GObject * object, guint prop_id,
 
 static void rsn_dvdsrc_finalize (GObject * object);
 
-static gboolean rsn_dvdsrc_start (RsnBaseSrc * bsrc);
-static gboolean rsn_dvdsrc_stop (RsnBaseSrc * bsrc);
-static gboolean rsn_dvdsrc_unlock (RsnBaseSrc * bsrc);
-static gboolean rsn_dvdsrc_unlock_stop (RsnBaseSrc * bsrc);
+static gboolean rsn_dvdsrc_start (GstBaseSrc * bsrc);
+static gboolean rsn_dvdsrc_stop (GstBaseSrc * bsrc);
+static gboolean rsn_dvdsrc_unlock (GstBaseSrc * bsrc);
+static gboolean rsn_dvdsrc_unlock_stop (GstBaseSrc * bsrc);
 
-static gboolean rsn_dvdsrc_is_seekable (RsnBaseSrc * bsrc);
-static gboolean rsn_dvdsrc_prepare_seek (RsnBaseSrc * bsrc, GstEvent * event,
+static gboolean rsn_dvdsrc_is_seekable (GstBaseSrc * bsrc);
+static gboolean rsn_dvdsrc_prepare_seek (GstBaseSrc * bsrc, GstEvent * event,
     GstSegment * segment);
-static gboolean rsn_dvdsrc_do_seek (RsnBaseSrc * bsrc, GstSegment * segment);
+static gboolean rsn_dvdsrc_do_seek (GstBaseSrc * bsrc, GstSegment * segment);
 static GstStateChangeReturn
 rsn_dvdsrc_change_state (GstElement * element, GstStateChange transition);
 
@@ -139,11 +140,11 @@ static void rsn_dvdsrc_check_nav_blocks (resinDvdSrc * src);
 static void rsn_dvdsrc_schedule_nav_cb (resinDvdSrc * src,
     RsnDvdPendingNav * next_nav);
 
-static gboolean rsn_dvdsrc_check_get_range (RsnBaseSrc * src);
-static GstFlowReturn rsn_dvdsrc_create (RsnBaseSrc * bsrc, guint64 offset,
+static gboolean rsn_dvdsrc_check_get_range (GstBaseSrc * src);
+static GstFlowReturn rsn_dvdsrc_create (GstBaseSrc * bsrc, guint64 offset,
     guint length, GstBuffer ** buf);
-static gboolean rsn_dvdsrc_src_event (RsnBaseSrc * basesrc, GstEvent * event);
-static gboolean rsn_dvdsrc_src_query (RsnBaseSrc * basesrc, GstQuery * query);
+static gboolean rsn_dvdsrc_src_event (GstBaseSrc * basesrc, GstEvent * event);
+static gboolean rsn_dvdsrc_src_query (GstBaseSrc * basesrc, GstQuery * query);
 
 static GstClockTime ifotime_to_gsttime (dvd_time_t * ifo_time);
 static void rsn_dvdsrc_send_commands_changed (resinDvdSrc * src);
@@ -188,17 +189,13 @@ rsn_dvdsrc_register_extra (GType rsn_dvdsrc_type)
 static void
 rsn_dvdsrc_base_init (gpointer gclass)
 {
-  static GstElementDetails element_details = {
-    "Resin DVD Src",
-    "Source/DVD",
-    "DVD source element",
-    "Jan Schmidt <thaytan@noraisin.net>"
-  };
+
   GstElementClass *element_class = GST_ELEMENT_CLASS (gclass);
 
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&src_factory));
-  gst_element_class_set_details (element_class, &element_details);
+  gst_element_class_set_details_simple (element_class, "Resin DVD Src",
+      "Source/DVD", "DVD source element", "Jan Schmidt <thaytan@noraisin.net>");
 }
 
 static void
@@ -206,11 +203,11 @@ rsn_dvdsrc_class_init (resinDvdSrcClass * klass)
 {
   GObjectClass *gobject_class;
   GstElementClass *gstelement_class;
-  RsnBaseSrcClass *gstbasesrc_class;
+  GstBaseSrcClass *gstbasesrc_class;
 
   gobject_class = (GObjectClass *) klass;
   gstelement_class = (GstElementClass *) klass;
-  gstbasesrc_class = RSN_BASE_SRC_CLASS (klass);
+  gstbasesrc_class = GST_BASE_SRC_CLASS (klass);
 
   gobject_class->finalize = rsn_dvdsrc_finalize;
   gobject_class->set_property = rsn_dvdsrc_set_property;
@@ -260,7 +257,7 @@ rsn_dvdsrc_init (resinDvdSrc * rsndvdsrc, resinDvdSrcClass * gclass)
   rsndvdsrc->branching = FALSE;
   rsndvdsrc->still_cond = g_cond_new ();
 
-  gst_base_src_set_format (RSN_BASE_SRC (rsndvdsrc), GST_FORMAT_TIME);
+  gst_base_src_set_format (GST_BASE_SRC (rsndvdsrc), GST_FORMAT_TIME);
 }
 
 static void
@@ -279,7 +276,7 @@ rsn_dvdsrc_finalize (GObject * object)
 }
 
 static gboolean
-rsn_dvdsrc_unlock (RsnBaseSrc * bsrc)
+rsn_dvdsrc_unlock (GstBaseSrc * bsrc)
 {
   resinDvdSrc *src = RESINDVDSRC (bsrc);
 
@@ -292,7 +289,7 @@ rsn_dvdsrc_unlock (RsnBaseSrc * bsrc)
 }
 
 static gboolean
-rsn_dvdsrc_unlock_stop (RsnBaseSrc * bsrc)
+rsn_dvdsrc_unlock_stop (GstBaseSrc * bsrc)
 {
   resinDvdSrc *src = RESINDVDSRC (bsrc);
 
@@ -354,7 +351,7 @@ rsn_dvdsrc_get_property (GObject * object, guint prop_id,
 }
 
 static gboolean
-rsn_dvdsrc_start (RsnBaseSrc * bsrc)
+rsn_dvdsrc_start (GstBaseSrc * bsrc)
 {
   resinDvdSrc *src = RESINDVDSRC (bsrc);
 
@@ -511,7 +508,7 @@ get_vts_attr (resinDvdSrc * src, gint n)
 }
 
 static gboolean
-rsn_dvdsrc_stop (RsnBaseSrc * bsrc)
+rsn_dvdsrc_stop (GstBaseSrc * bsrc)
 {
   resinDvdSrc *src = RESINDVDSRC (bsrc);
   gboolean ret = TRUE;
@@ -597,9 +594,8 @@ rsn_dvdsrc_do_still (resinDvdSrc * src, int duration)
   GstEvent *still_event;
   GstEvent *hl_event;
   gboolean cmds_changed;
-  GstStructure *s;
   GstEvent *seg_event;
-  GstSegment *segment = &(RSN_BASE_SRC (src)->segment);
+  GstSegment *segment = &(GST_BASE_SRC (src)->segment);
 
   if (src->in_still_state == FALSE) {
     GST_DEBUG_OBJECT (src, "**** Start STILL FRAME. Duration %d ****",
@@ -610,12 +606,9 @@ rsn_dvdsrc_do_still (resinDvdSrc * src, int duration)
     else
       src->still_time_remaining = GST_SECOND * duration;
 
-    /* Send a close-segment event, and a dvd-still start
+    /* Send a close-segment event, and a still-frame start
      * event, then sleep */
-    s = gst_structure_new ("application/x-gst-dvd",
-        "event", G_TYPE_STRING, "dvd-still",
-        "still-state", G_TYPE_BOOLEAN, TRUE, NULL);
-    still_event = gst_event_new_custom (GST_EVENT_CUSTOM_DOWNSTREAM, s);
+    still_event = gst_video_event_new_still_frame (TRUE);
 
     gst_segment_set_last_stop (segment, GST_FORMAT_TIME, src->cur_end_ts);
 
@@ -632,11 +625,11 @@ rsn_dvdsrc_do_still (resinDvdSrc * src, int duration)
     /* Now, send the events. We need to drop the dvd lock while doing so,
      * and then check after if we got flushed */
     g_mutex_unlock (src->dvd_lock);
-    gst_pad_push_event (RSN_BASE_SRC_PAD (src), still_event);
-    gst_pad_push_event (RSN_BASE_SRC_PAD (src), seg_event);
+    gst_pad_push_event (GST_BASE_SRC_PAD (src), still_event);
+    gst_pad_push_event (GST_BASE_SRC_PAD (src), seg_event);
     if (hl_event) {
       GST_LOG_OBJECT (src, "Sending highlight event before still");
-      gst_pad_push_event (RSN_BASE_SRC_PAD (src), hl_event);
+      gst_pad_push_event (GST_BASE_SRC_PAD (src), hl_event);
     }
     if (cmds_changed)
       rsn_dvdsrc_send_commands_changed (src);
@@ -734,10 +727,7 @@ rsn_dvdsrc_do_still (resinDvdSrc * src, int duration)
 
     /* Tell downstream the still is over.
      * We only do this if the still isn't interrupted: */
-    s = gst_structure_new ("application/x-gst-dvd",
-        "event", G_TYPE_STRING, "dvd-still",
-        "still-state", G_TYPE_BOOLEAN, FALSE, NULL);
-    still_event = gst_event_new_custom (GST_EVENT_CUSTOM_DOWNSTREAM, s);
+    still_event = gst_video_event_new_still_frame (FALSE);
 
     /* If the segment was too short in a timed still, it may need extending */
     if (segment->last_stop < segment->start + GST_SECOND * duration)
@@ -745,7 +735,7 @@ rsn_dvdsrc_do_still (resinDvdSrc * src, int duration)
           segment->start + (GST_SECOND * duration));
 
     g_mutex_unlock (src->dvd_lock);
-    gst_pad_push_event (RSN_BASE_SRC_PAD (src), still_event);
+    gst_pad_push_event (GST_BASE_SRC_PAD (src), still_event);
     g_mutex_lock (src->dvd_lock);
   }
 
@@ -876,8 +866,10 @@ rsn_dvdsrc_step (resinDvdSrc * src, gboolean have_dvd_lock)
   gint event, len;
 
   /* Allocate an output buffer if there isn't a pending one */
-  if (src->alloc_buf == NULL)
+  if (src->alloc_buf == NULL) {
     src->alloc_buf = gst_buffer_new_and_alloc (DVD_VIDEO_LB_LEN);
+    gst_buffer_set_caps (src->alloc_buf, GST_PAD_CAPS (GST_BASE_SRC_PAD (src)));
+  }
 
   data = GST_BUFFER_DATA (src->alloc_buf);
   len = DVD_VIDEO_LB_LEN;
@@ -1095,7 +1087,7 @@ rsn_dvdsrc_step (resinDvdSrc * src, gboolean have_dvd_lock)
     g_mutex_unlock (src->dvd_lock);
     GST_DEBUG_OBJECT (src, "Sending highlight event - button %d",
         src->active_button);
-    gst_pad_push_event (RSN_BASE_SRC_PAD (src), hl_event);
+    gst_pad_push_event (GST_BASE_SRC_PAD (src), hl_event);
     g_mutex_lock (src->dvd_lock);
   }
 
@@ -1229,7 +1221,7 @@ rsn_dvdsrc_prepare_next_block (resinDvdSrc * src, gboolean have_dvd_lock)
 }
 
 static gboolean
-rsn_dvdsrc_check_get_range (RsnBaseSrc * src)
+rsn_dvdsrc_check_get_range (GstBaseSrc * src)
 {
   /* ResinDVD never operates in pull mode. There might be
    * a reason to in the future though? */
@@ -1237,11 +1229,11 @@ rsn_dvdsrc_check_get_range (RsnBaseSrc * src)
 }
 
 static GstFlowReturn
-rsn_dvdsrc_create (RsnBaseSrc * bsrc, guint64 offset,
+rsn_dvdsrc_create (GstBaseSrc * bsrc, guint64 offset,
     guint length, GstBuffer ** outbuf)
 {
   resinDvdSrc *src = RESINDVDSRC (bsrc);
-  GstSegment *segment = &(RSN_BASE_SRC (src)->segment);
+  GstSegment *segment = &(GST_BASE_SRC (src)->segment);
   GstFlowReturn ret;
   GstEvent *streams_event = NULL;
   GstEvent *clut_event = NULL;
@@ -1292,20 +1284,20 @@ rsn_dvdsrc_create (RsnBaseSrc * bsrc, guint64 offset,
    * we change segment */
   if (streams_event) {
     GST_LOG_OBJECT (src, "Pushing stream layout event");
-    gst_pad_push_event (RSN_BASE_SRC_PAD (src), streams_event);
+    gst_pad_push_event (GST_BASE_SRC_PAD (src), streams_event);
   }
   if (clut_event) {
     GST_LOG_OBJECT (src, "Pushing clut event");
-    gst_pad_push_event (RSN_BASE_SRC_PAD (src), clut_event);
+    gst_pad_push_event (GST_BASE_SRC_PAD (src), clut_event);
   }
   /* Out of band events */
   if (spu_select_event) {
     GST_LOG_OBJECT (src, "Pushing spu_select event");
-    gst_pad_push_event (RSN_BASE_SRC_PAD (src), spu_select_event);
+    gst_pad_push_event (GST_BASE_SRC_PAD (src), spu_select_event);
   }
   if (audio_select_event) {
     GST_LOG_OBJECT (src, "Pushing audio_select event");
-    gst_pad_push_event (RSN_BASE_SRC_PAD (src), audio_select_event);
+    gst_pad_push_event (GST_BASE_SRC_PAD (src), audio_select_event);
   }
 
   if (src->need_segment) {
@@ -1323,7 +1315,7 @@ rsn_dvdsrc_create (RsnBaseSrc * bsrc, guint64 offset,
         GST_TIME_ARGS (src->cur_start_ts), GST_TIME_ARGS (src->cur_end_ts),
         GST_TIME_ARGS (src->cur_vobu_base_ts), GST_TIME_ARGS (position));
 
-    gst_base_src_new_seamless_segment (RSN_BASE_SRC (src),
+    gst_base_src_new_seamless_segment (GST_BASE_SRC (src),
         src->cur_start_ts, -1, position);
 
     src->need_segment = FALSE;
@@ -1363,7 +1355,7 @@ rsn_dvdsrc_create (RsnBaseSrc * bsrc, guint64 offset,
   if (highlight_event) {
     GST_LOG_OBJECT (src, "Pushing highlight event with TS %" GST_TIME_FORMAT,
         GST_TIME_ARGS (GST_EVENT_TIMESTAMP (highlight_event)));
-    gst_pad_push_event (RSN_BASE_SRC_PAD (src), highlight_event);
+    gst_pad_push_event (GST_BASE_SRC_PAD (src), highlight_event);
   }
 
   if (angles_msg) {
@@ -1712,7 +1704,7 @@ rsn_dvdsrc_handle_navigation_event (resinDvdSrc * src, GstEvent * event)
       if (hl_event) {
         GST_DEBUG_OBJECT (src, "Sending highlight change event - button: %d",
             src->active_button);
-        gst_pad_push_event (RSN_BASE_SRC_PAD (src), hl_event);
+        gst_pad_push_event (GST_BASE_SRC_PAD (src), hl_event);
       }
 
       /* Send ourselves a seek event to wake everything up and flush */
@@ -1750,7 +1742,7 @@ rsn_dvdsrc_handle_navigation_event (resinDvdSrc * src, GstEvent * event)
     if (hl_event) {
       GST_DEBUG_OBJECT (src, "Sending highlight change event - button: %d",
           src->active_button);
-      gst_pad_push_event (RSN_BASE_SRC_PAD (src), hl_event);
+      gst_pad_push_event (GST_BASE_SRC_PAD (src), hl_event);
     }
 
     if (cmds_changed)
@@ -2153,7 +2145,7 @@ rsn_dvdsrc_enqueue_nav_block (resinDvdSrc * src, GstBuffer * nav_buf,
     GstClockTime ts)
 {
   RsnDvdPendingNav *pend_nav = g_new0 (RsnDvdPendingNav, 1);
-  GstSegment *seg = &(RSN_BASE_SRC (src)->segment);
+  GstSegment *seg = &(GST_BASE_SRC (src)->segment);
 
   pend_nav->buffer = gst_buffer_ref (nav_buf);
   pend_nav->ts = ts;
@@ -2327,7 +2319,7 @@ rsn_dvdsrc_check_nav_blocks (resinDvdSrc * src)
 }
 
 static gboolean
-rsn_dvdsrc_src_event (RsnBaseSrc * basesrc, GstEvent * event)
+rsn_dvdsrc_src_event (GstBaseSrc * basesrc, GstEvent * event)
 {
   resinDvdSrc *src = RESINDVDSRC (basesrc);
   gboolean res;
@@ -2346,13 +2338,13 @@ rsn_dvdsrc_src_event (RsnBaseSrc * basesrc, GstEvent * event)
       GST_DEBUG_OBJECT (src, "%s seek event",
           src->flushing_seek ? "flushing" : "non-flushing");
 
-      res = RSN_BASE_SRC_CLASS (parent_class)->event (basesrc, event);
+      res = GST_BASE_SRC_CLASS (parent_class)->event (basesrc, event);
       break;
     }
     default:
       GST_LOG_OBJECT (src, "handling %s event", GST_EVENT_TYPE_NAME (event));
 
-      res = RSN_BASE_SRC_CLASS (parent_class)->event (basesrc, event);
+      res = GST_BASE_SRC_CLASS (parent_class)->event (basesrc, event);
       break;
   }
 
@@ -2403,7 +2395,7 @@ rsn_dvdsrc_change_state (GstElement * element, GstStateChange transition)
 }
 
 static gboolean
-rsn_dvdsrc_src_query (RsnBaseSrc * basesrc, GstQuery * query)
+rsn_dvdsrc_src_query (GstBaseSrc * basesrc, GstQuery * query)
 {
   resinDvdSrc *src = RESINDVDSRC (basesrc);
   gboolean res = FALSE;
@@ -2484,11 +2476,11 @@ rsn_dvdsrc_src_query (RsnBaseSrc * basesrc, GstQuery * query)
       if (nq_type != GST_NAVIGATION_QUERY_INVALID)
         res = rsn_dvdsrc_handle_navigation_query (src, nq_type, query);
       else
-        res = RSN_BASE_SRC_CLASS (parent_class)->query (basesrc, query);
+        res = GST_BASE_SRC_CLASS (parent_class)->query (basesrc, query);
       break;
     }
     default:
-      res = RSN_BASE_SRC_CLASS (parent_class)->query (basesrc, query);
+      res = GST_BASE_SRC_CLASS (parent_class)->query (basesrc, query);
       break;
   }
 
@@ -2496,13 +2488,13 @@ rsn_dvdsrc_src_query (RsnBaseSrc * basesrc, GstQuery * query)
 }
 
 static gboolean
-rsn_dvdsrc_is_seekable (RsnBaseSrc * bsrc)
+rsn_dvdsrc_is_seekable (GstBaseSrc * bsrc)
 {
   return TRUE;
 }
 
 static gboolean
-rsn_dvdsrc_prepare_seek (RsnBaseSrc * bsrc, GstEvent * event,
+rsn_dvdsrc_prepare_seek (GstBaseSrc * bsrc, GstEvent * event,
     GstSegment * segment)
 {
   GstSeekType cur_type, stop_type;
@@ -2532,7 +2524,7 @@ rsn_dvdsrc_prepare_seek (RsnBaseSrc * bsrc, GstEvent * event,
   }
 
   /* Let basesrc handle other formats */
-  ret = RSN_BASE_SRC_CLASS (parent_class)->prepare_seek_segment (bsrc,
+  ret = GST_BASE_SRC_CLASS (parent_class)->prepare_seek_segment (bsrc,
       event, segment);
 
   return ret;
@@ -2591,6 +2583,9 @@ rsn_dvdsrc_get_sector_from_time_tmap (resinDvdSrc * src, GstClockTime ts)
 
   /* Get the time map */
   title_tmap = vts_tmapt->tmap + vts_ttn - 1;
+  if (title_tmap->tmu == 0)
+    return -1;
+
   entry = ts / (title_tmap->tmu * GST_SECOND);
   if (entry == 0)
     return 0;
@@ -2658,7 +2653,7 @@ rsn_dvdsrc_seek_to_time (resinDvdSrc * src, GstClockTime ts)
 }
 
 static gboolean
-rsn_dvdsrc_do_seek (RsnBaseSrc * bsrc, GstSegment * segment)
+rsn_dvdsrc_do_seek (GstBaseSrc * bsrc, GstSegment * segment)
 {
   resinDvdSrc *src = RESINDVDSRC (bsrc);
   gboolean ret = FALSE;
@@ -2787,14 +2782,3 @@ fail:
   GST_DEBUG_OBJECT (src, "Seek in format %d failed", segment->format);
   return FALSE;
 }
-
-gboolean
-rsndvdsrc_init (GstPlugin * plugin)
-{
-  gboolean res;
-
-  res = gst_element_register (plugin, "rsndvdsrc",
-      GST_RANK_NONE, RESIN_TYPE_DVDSRC);
-
-  return res;
-}
