@@ -63,6 +63,8 @@ struct _GstSchroDecClass
   GstBaseVideoDecoder base_video_decoder_class;
 };
 
+GType gst_schro_dec_get_type (void);
+
 
 /* GstSchroDec signals and args */
 enum
@@ -116,11 +118,7 @@ GST_BOILERPLATE (GstSchroDec, gst_schro_dec, GstBaseVideoDecoder,
 static void
 gst_schro_dec_base_init (gpointer g_class)
 {
-  static GstElementDetails compress_details =
-      GST_ELEMENT_DETAILS ("Dirac Decoder",
-      "Codec/Decoder/Video",
-      "Decode Dirac streams",
-      "David Schleef <ds@schleef.org>");
+
   GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
 
   gst_element_class_add_pad_template (element_class,
@@ -128,7 +126,9 @@ gst_schro_dec_base_init (gpointer g_class)
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&gst_schro_dec_sink_template));
 
-  gst_element_class_set_details (element_class, &compress_details);
+  gst_element_class_set_details_simple (element_class, "Dirac Decoder",
+      "Codec/Decoder/Video",
+      "Decode Dirac streams", "David Schleef <ds@schleef.org>");
 }
 
 static void
