@@ -1,6 +1,6 @@
 /*
  * gstwildmidi - wildmidi plugin for gstreamer
- * 
+ *
  * Copyright 2007 Wouter Paesen <wouter@blue-gate.be>
  *
  * This library is free software; you can redistribute it and/or
@@ -23,21 +23,17 @@
  * SECTION:element-wildmidi
  * @see_also: timidity
  *
- * <refsect2>
- * <para>
  * This element renders midi-files as audio streams using
  * <ulink url="http://wildmidi.sourceforge.net//">Wildmidi</ulink>.
  * It offers better sound quality compared to the timidity element. Wildmidi
  * uses the same sound-patches as timidity (it tries the path in $WILDMIDI_CFG,
  * $HOME/.wildmidirc and /etc/wildmidi.cfg)
- * </para>
- * <para>
- * </para>
+ *
+ * <refsect2>
  * <title>Example pipeline</title>
- * <programlisting>
+ * |[
  * gst-launch filesrc location=song.mid ! wildmidi ! alsasink
- * </programlisting>
- * This example pipeline will parse the midi and render to raw audio which is
+ * ]| This example pipeline will parse the midi and render to raw audio which is
  * played via alsa.
  * </refsect2>
  */
@@ -65,12 +61,6 @@
 GST_DEBUG_CATEGORY_STATIC (gst_wildmidi_debug);
 #define GST_CAT_DEFAULT gst_wildmidi_debug
 
-static const GstElementDetails gst_wildmidi_details =
-GST_ELEMENT_DETAILS ("WildMidi",
-    "Codec/Decoder/Audio",
-    "Midi Synthesizer Element",
-    "Wouter Paesen <wouter@blue-gate.be>");
-
 enum
 {
   /* FILL ME */
@@ -85,15 +75,19 @@ enum
   /* FILL ME */
 };
 
-static void gst_wildmidi_base_init (gpointer g_class);
-static void gst_wildmidi_class_init (GstWildmidiClass * klass);
+static void gst_wildmidi_finalize (GObject * object);
 
+static gboolean gst_wildmidi_sink_event (GstPad * pad, GstEvent * event);
 static gboolean gst_wildmidi_src_event (GstPad * pad, GstEvent * event);
+
 static GstStateChangeReturn gst_wildmidi_change_state (GstElement * element,
     GstStateChange transition);
 static gboolean gst_wildmidi_activate (GstPad * pad);
 static gboolean gst_wildmidi_activatepull (GstPad * pad, gboolean active);
+
 static void gst_wildmidi_loop (GstPad * sinkpad);
+static GstFlowReturn gst_wildmidi_chain (GstPad * sinkpad, GstBuffer * buffer);
+
 static gboolean gst_wildmidi_src_query (GstPad * pad, GstQuery * query);
 
 static void gst_wildmidi_set_property (GObject * object, guint prop_id,
@@ -104,7 +98,7 @@ static void gst_wildmidi_get_property (GObject * object, guint prop_id,
 static GstStaticPadTemplate sink_factory = GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
     GST_PAD_ALWAYS,
-    GST_STATIC_CAPS ("audio/midi")
+    GST_STATIC_CAPS ("audio/midi; audio/riff-midi")
     );
 
 static GstStaticPadTemplate src_factory = GST_STATIC_PAD_TEMPLATE ("src",
@@ -127,11 +121,13 @@ gst_wildmidi_base_init (gpointer gclass)
       gst_static_pad_template_get (&src_factory));
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&sink_factory));
-  gst_element_class_set_details (element_class, &gst_wildmidi_details);
+  gst_element_class_set_details_simple (element_class, "WildMidi",
+      "Codec/Decoder/Audio",
+      "Midi Synthesizer Element", "Wouter Paesen <wouter@blue-gate.be>");
 }
 
 static gboolean
-wildmidi_open_config ()
+wildmidi_open_config (void)
 {
   gchar *path = g_strdup (g_getenv ("WILDMIDI_CFG"));
   gint ret;
@@ -154,7 +150,20 @@ wildmidi_open_config ()
   }
 
   if (path == NULL) {
-    path = g_build_path (G_DIR_SEPARATOR_S, "/etc", "wildmidi.cfg", NULL);
+    path =
+        g_build_path (G_DIR_SEPARATOR_S, G_DIR_SEPARATOR_S "etc",
+        "wildmidi.cfg", NULL);
+    GST_DEBUG ("trying %s", path);
+    if (path && (g_access (path, R_OK) == -1)) {
+      g_free (path);
+      path = NULL;
+    }
+  }
+
+  if (path == NULL) {
+    path =
+        g_build_path (G_DIR_SEPARATOR_S, G_DIR_SEPARATOR_S "etc", "wildmidi",
+        "wildmidi.cfg", NULL);
     GST_DEBUG ("trying %s", path);
     if (path && (g_access (path, R_OK) == -1)) {
       g_free (path);
@@ -172,7 +181,20 @@ wildmidi_open_config ()
   }
 
   if (path == NULL) {
-    path = g_build_path (G_DIR_SEPARATOR_S, "/etc", "timidity.cfg", NULL);
+    path =
+        g_build_path (G_DIR_SEPARATOR_S, G_DIR_SEPARATOR_S "etc",
+        "timidity.cfg", NULL);
+    GST_DEBUG ("trying %s", path);
+    if (path && (g_access (path, R_OK) == -1)) {
+      g_free (path);
+      path = NULL;
+    }
+  }
+
+  if (path == NULL) {
+    path =
+        g_build_path (G_DIR_SEPARATOR_S, G_DIR_SEPARATOR_S "etc", "timidity",
+        "timidity.cfg", NULL);
     GST_DEBUG ("trying %s", path);
     if (path && (g_access (path, R_OK) == -1)) {
       g_free (path);
@@ -185,7 +207,7 @@ wildmidi_open_config ()
      * ln -s /usr/share/timidity/timidity.cfg /etc/wildmidi.cfg
      * we could make it use : WILDMIDI_CFG
      * but unfortunately it fails to create a proper filename if the config
-     * has a redirect   
+     * has a redirect
      * http://sourceforge.net/tracker/index.php?func=detail&aid=1657358&group_id=42635&atid=433744
      */
     GST_WARNING ("no config file, can't initialise");
@@ -209,7 +231,7 @@ gst_wildmidi_class_init (GstWildmidiClass * klass)
   gobject_class = (GObjectClass *) klass;
   gstelement_class = (GstElementClass *) klass;
 
-  gstelement_class->change_state = gst_wildmidi_change_state;
+  gobject_class->finalize = gst_wildmidi_finalize;
   gobject_class->set_property = gst_wildmidi_set_property;
   gobject_class->get_property = gst_wildmidi_get_property;
 
@@ -220,6 +242,8 @@ gst_wildmidi_class_init (GstWildmidiClass * klass)
   g_object_class_install_property (gobject_class, ARG_HIGH_QUALITY,
       g_param_spec_boolean ("high-quality", "High Quality",
           "High Quality", TRUE, G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+
+  gstelement_class->change_state = gst_wildmidi_change_state;
 }
 
 /* initialize the new element
@@ -232,13 +256,6 @@ gst_wildmidi_init (GstWildmidi * filter, GstWildmidiClass * g_class)
 {
   GstElementClass *klass = GST_ELEMENT_GET_CLASS (filter);
 
-  /* initialise wildmidi library */
-  if (wildmidi_open_config ()) {
-    filter->initialized = TRUE;
-  } else {
-    GST_WARNING ("can't initialize wildmidi");
-  }
-
   filter->sinkpad =
       gst_pad_new_from_template (gst_element_class_get_pad_template (klass,
           "sink"), "sink");
@@ -246,8 +263,8 @@ gst_wildmidi_init (GstWildmidi * filter, GstWildmidiClass * g_class)
   gst_pad_set_activatepull_function (filter->sinkpad,
       gst_wildmidi_activatepull);
   gst_pad_set_activate_function (filter->sinkpad, gst_wildmidi_activate);
-  gst_pad_set_setcaps_function (filter->sinkpad, gst_pad_set_caps);
-  gst_pad_use_fixed_caps (filter->sinkpad);
+  gst_pad_set_event_function (filter->sinkpad, gst_wildmidi_sink_event);
+  gst_pad_set_chain_function (filter->sinkpad, gst_wildmidi_chain);
   gst_element_add_pad (GST_ELEMENT (filter), filter->sinkpad);
 
   filter->srcpad =
@@ -257,14 +274,26 @@ gst_wildmidi_init (GstWildmidi * filter, GstWildmidiClass * g_class)
   gst_pad_set_query_function (filter->srcpad, gst_wildmidi_src_query);
   gst_pad_set_event_function (filter->srcpad, gst_wildmidi_src_event);
   gst_pad_use_fixed_caps (filter->srcpad);
-  gst_pad_set_setcaps_function (filter->srcpad, gst_pad_set_caps);
 
   gst_element_add_pad (GST_ELEMENT (filter), filter->srcpad);
 
   gst_segment_init (filter->o_segment, GST_FORMAT_DEFAULT);
 
+  filter->adapter = gst_adapter_new ();
+
   filter->bytes_per_frame = WILDMIDI_BPS;
-  filter->time_per_frame = GST_SECOND / WILDMIDI_RATE;
+}
+
+static void
+gst_wildmidi_finalize (GObject * object)
+{
+  GstWildmidi *wildmidi;
+
+  wildmidi = GST_WILDMIDI (object);
+
+  g_object_unref (wildmidi->adapter);
+
+  G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
 static gboolean
@@ -275,14 +304,14 @@ gst_wildmidi_src_convert (GstWildmidi * wildmidi,
   gboolean res = TRUE;
   gint64 frames;
 
-  if (src_format == *dest_format) {
+  if (src_format == *dest_format || src_value == -1) {
     *dest_value = src_value;
     goto done;
   }
 
   switch (src_format) {
     case GST_FORMAT_TIME:
-      frames = src_value / wildmidi->time_per_frame;
+      frames = gst_util_uint64_scale_int (src_value, WILDMIDI_RATE, GST_SECOND);
       break;
     case GST_FORMAT_BYTES:
       frames = src_value / (wildmidi->bytes_per_frame);
@@ -297,7 +326,8 @@ gst_wildmidi_src_convert (GstWildmidi * wildmidi,
 
   switch (*dest_format) {
     case GST_FORMAT_TIME:
-      *dest_value = frames * wildmidi->time_per_frame;
+      *dest_value =
+          gst_util_uint64_scale_int (frames, GST_SECOND, WILDMIDI_RATE);
       break;
     case GST_FORMAT_BYTES:
       *dest_value = frames * wildmidi->bytes_per_frame;
@@ -330,11 +360,13 @@ gst_wildmidi_src_query (GstPad * pad, GstQuery * query)
   switch (GST_QUERY_TYPE (query)) {
     case GST_QUERY_DURATION:
       gst_query_set_duration (query, GST_FORMAT_TIME,
-          wildmidi->o_len * wildmidi->time_per_frame);
+          gst_util_uint64_scale_int (wildmidi->o_len, GST_SECOND,
+              WILDMIDI_RATE));
       break;
     case GST_QUERY_POSITION:
       gst_query_set_position (query, GST_FORMAT_TIME,
-          wildmidi->o_segment->last_stop * wildmidi->time_per_frame);
+          gst_util_uint64_scale_int (wildmidi->o_segment->last_stop, GST_SECOND,
+              WILDMIDI_RATE));
       break;
     case GST_QUERY_CONVERT:
       gst_query_parse_convert (query, &src_format, &src_value,
@@ -370,68 +402,115 @@ gst_wildmidi_src_query (GstPad * pad, GstQuery * query)
   return res;
 }
 
-static gboolean
-gst_wildmidi_get_upstream_size (GstWildmidi * wildmidi, gint64 * size)
-{
-  GstFormat format = GST_FORMAT_BYTES;
-  gboolean res = FALSE;
-  GstPad *peer = gst_pad_get_peer (wildmidi->sinkpad);
-
-  if (peer != NULL)
-    res = gst_pad_query_duration (peer, &format, size) && *size >= 0;
-
-  gst_object_unref (peer);
-  return res;
-}
-
-static GstSegment *
-gst_wildmidi_get_segment (GstWildmidi * wildmidi, GstFormat format,
-    gboolean update)
+static GstEvent *
+gst_wildmidi_get_new_segment_event (GstWildmidi * wildmidi, GstFormat format)
 {
   gint64 start, stop, time;
-
-  GstSegment *segment = gst_segment_new ();
-
-  gst_wildmidi_src_convert (wildmidi,
-      wildmidi->o_segment->format, wildmidi->o_segment->start, &format, &start);
-
-  if (wildmidi->o_segment->stop == GST_CLOCK_TIME_NONE) {
-    stop = GST_CLOCK_TIME_NONE;
-  } else {
-    gst_wildmidi_src_convert (wildmidi,
-        wildmidi->o_segment->format, wildmidi->o_segment->stop, &format, &stop);
-  }
-
-  gst_wildmidi_src_convert (wildmidi,
-      wildmidi->o_segment->format, wildmidi->o_segment->time, &format, &time);
-
-  gst_segment_set_newsegment_full (segment, update,
-      wildmidi->o_segment->rate, wildmidi->o_segment->applied_rate,
-      format, start, stop, time);
-
-  segment->last_stop = time;
-
-  return segment;
-}
-
-static GstEvent *
-gst_wildmidi_get_new_segment_event (GstWildmidi * wildmidi, GstFormat format,
-    gboolean update)
-{
   GstSegment *segment;
   GstEvent *event;
+  GstFormat src_format;
 
-  segment = gst_wildmidi_get_segment (wildmidi, format, update);
+  segment = wildmidi->o_segment;
+  src_format = segment->format;
 
-  event = gst_event_new_new_segment_full (update,
-      segment->rate, segment->applied_rate, segment->format,
-      segment->start, segment->stop, segment->time);
+  /* convert the segment values to the target format */
+  gst_wildmidi_src_convert (wildmidi, src_format, segment->start, &format,
+      &start);
+  gst_wildmidi_src_convert (wildmidi, src_format, segment->stop, &format,
+      &stop);
+  gst_wildmidi_src_convert (wildmidi, src_format, segment->time, &format,
+      &time);
 
-  gst_segment_free (segment);
+  event = gst_event_new_new_segment_full (FALSE,
+      segment->rate, segment->applied_rate, format, start, stop, time);
 
   return event;
 }
 
+static gboolean
+gst_wildmidi_do_seek (GstWildmidi * wildmidi, GstEvent * event)
+{
+  gdouble rate;
+  GstFormat src_format, dst_format;
+  GstSeekFlags flags;
+  GstSeekType start_type, stop_type;
+  gint64 start, stop;
+  gboolean flush, update, accurate;
+  gboolean res;
+  unsigned long int sample;
+  GstSegment *segment;
+
+  if (!wildmidi->song)
+    return FALSE;
+
+  gst_event_parse_seek (event, &rate, &src_format, &flags,
+      &start_type, &start, &stop_type, &stop);
+
+  /* convert the input format to samples */
+  dst_format = GST_FORMAT_DEFAULT;
+  res = TRUE;
+  if (start_type != GST_SEEK_TYPE_NONE) {
+    res =
+        gst_wildmidi_src_convert (wildmidi, src_format, start, &dst_format,
+        &start);
+  }
+  if (res && stop_type != GST_SEEK_TYPE_NONE) {
+    res =
+        gst_wildmidi_src_convert (wildmidi, src_format, stop, &dst_format,
+        &stop);
+  }
+  /* unsupported format */
+  if (!res)
+    return res;
+
+  flush = ((flags & GST_SEEK_FLAG_FLUSH) == GST_SEEK_FLAG_FLUSH);
+  accurate = ((flags & GST_SEEK_FLAG_ACCURATE) == GST_SEEK_FLAG_ACCURATE);
+
+  if (flush) {
+    GST_DEBUG ("performing flush");
+    gst_pad_push_event (wildmidi->srcpad, gst_event_new_flush_start ());
+  } else {
+    gst_pad_stop_task (wildmidi->sinkpad);
+  }
+
+  segment = wildmidi->o_segment;
+
+  GST_PAD_STREAM_LOCK (wildmidi->sinkpad);
+
+  if (flush) {
+    gst_pad_push_event (wildmidi->srcpad, gst_event_new_flush_stop ());
+  }
+
+  /* update the segment now */
+  gst_segment_set_seek (segment, rate, dst_format, flags,
+      start_type, start, stop_type, stop, &update);
+
+  /* we need to seek to last_stop in the segment now, sample will be updated */
+  sample = segment->last_stop;
+
+  GST_OBJECT_LOCK (wildmidi);
+  if (accurate) {
+    WildMidi_SampledSeek (wildmidi->song, &sample);
+  } else {
+    WildMidi_FastSeek (wildmidi->song, &sample);
+  }
+  GST_OBJECT_UNLOCK (wildmidi);
+
+  segment->start = segment->time = segment->last_stop = sample;
+
+  gst_pad_push_event (wildmidi->srcpad,
+      gst_wildmidi_get_new_segment_event (wildmidi, GST_FORMAT_TIME));
+
+  gst_pad_start_task (wildmidi->sinkpad,
+      (GstTaskFunction) gst_wildmidi_loop, wildmidi->sinkpad);
+
+  wildmidi->discont = TRUE;
+  GST_PAD_STREAM_UNLOCK (wildmidi->sinkpad);
+  GST_DEBUG ("seek done");
+
+  return TRUE;
+}
+
 static gboolean
 gst_wildmidi_src_event (GstPad * pad, GstEvent * event)
 {
@@ -442,66 +521,7 @@ gst_wildmidi_src_event (GstPad * pad, GstEvent * event)
 
   switch (GST_EVENT_TYPE (event)) {
     case GST_EVENT_SEEK:
-    {
-      gdouble rate;
-      GstFormat src_format, dst_format;
-      GstSeekFlags flags;
-      GstSeekType start_type, stop_type;
-      gint64 orig_start, start, stop;
-      gboolean flush, update;
-
-      if (!wildmidi->song)
-        break;
-
-      gst_event_parse_seek (event, &rate, &src_format, &flags,
-          &start_type, &orig_start, &stop_type, &stop);
-
-      dst_format = GST_FORMAT_DEFAULT;
-
-      gst_wildmidi_src_convert (wildmidi, src_format, orig_start,
-          &dst_format, &start);
-      gst_wildmidi_src_convert (wildmidi, src_format, stop, &dst_format, &stop);
-
-      flush = ((flags & GST_SEEK_FLAG_FLUSH) == GST_SEEK_FLAG_FLUSH);
-
-      if (flush) {
-        GST_DEBUG ("performing flush");
-        gst_pad_push_event (wildmidi->srcpad, gst_event_new_flush_start ());
-      } else {
-        gst_pad_stop_task (wildmidi->sinkpad);
-      }
-
-      GST_PAD_STREAM_LOCK (wildmidi->sinkpad);
-
-      if (flush) {
-        gst_pad_push_event (wildmidi->srcpad, gst_event_new_flush_stop ());
-      }
-
-      gst_segment_set_seek (wildmidi->o_segment, rate, dst_format, flags,
-          start_type, start, stop_type, stop, &update);
-
-      if ((flags && GST_SEEK_FLAG_SEGMENT) == GST_SEEK_FLAG_SEGMENT) {
-        GST_DEBUG_OBJECT (wildmidi, "received segment seek %d, %d",
-            (gint) start_type, (gint) stop_type);
-      } else {
-        GST_DEBUG_OBJECT (wildmidi, "received normal seek %d",
-            (gint) start_type);
-        update = FALSE;
-      }
-
-      gst_pad_push_event (wildmidi->srcpad,
-          gst_wildmidi_get_new_segment_event (wildmidi, GST_FORMAT_TIME,
-              update));
-
-      wildmidi->o_seek = TRUE;
-
-      gst_pad_start_task (wildmidi->sinkpad,
-          (GstTaskFunction) gst_wildmidi_loop, wildmidi->sinkpad);
-
-      GST_PAD_STREAM_UNLOCK (wildmidi->sinkpad);
-      GST_DEBUG ("seek done");
-    }
-      res = TRUE;
+      res = gst_wildmidi_do_seek (wildmidi, event);
       break;
     default:
       break;
@@ -511,13 +531,14 @@ gst_wildmidi_src_event (GstPad * pad, GstEvent * event)
   return res;
 }
 
+
 static gboolean
 gst_wildmidi_activate (GstPad * sinkpad)
 {
   if (gst_pad_check_pull_range (sinkpad))
     return gst_pad_activate_pull (sinkpad, TRUE);
 
-  return FALSE;
+  return gst_pad_activate_push (sinkpad, TRUE);
 }
 
 static gboolean
@@ -530,42 +551,44 @@ gst_wildmidi_activatepull (GstPad * pad, gboolean active)
   }
 }
 
-static GstBuffer *
-gst_wildmidi_allocate_buffer (GstWildmidi * wildmidi, gint64 samples)
-{
-  return gst_buffer_new_and_alloc (samples * wildmidi->bytes_per_frame);
-}
-
 static GstBuffer *
 gst_wildmidi_clip_buffer (GstWildmidi * wildmidi, GstBuffer * buffer)
 {
+  gint64 start, stop;
   gint64 new_start, new_stop;
   gint64 offset, length;
   GstBuffer *out;
+  guint64 bpf;
 
+  /* clipping disabled for now */
   return buffer;
 
+  start = GST_BUFFER_OFFSET (buffer);
+  stop = GST_BUFFER_OFFSET_END (buffer);
+
   if (!gst_segment_clip (wildmidi->o_segment, GST_FORMAT_DEFAULT,
-          GST_BUFFER_OFFSET (buffer), GST_BUFFER_OFFSET_END (buffer),
-          &new_start, &new_stop)) {
+          start, stop, &new_start, &new_stop)) {
     gst_buffer_unref (buffer);
     return NULL;
   }
 
-  if (GST_BUFFER_OFFSET (buffer) == new_start &&
-      GST_BUFFER_OFFSET_END (buffer) == new_stop)
+  if (start == new_start && stop == new_stop)
     return buffer;
 
-  offset = new_start - GST_BUFFER_OFFSET (buffer);
+
+  offset = new_start - start;
   length = new_stop - new_start;
 
-  out = gst_buffer_create_sub (buffer, offset * wildmidi->bytes_per_frame,
-      length * wildmidi->bytes_per_frame);
+  bpf = wildmidi->bytes_per_frame;
+  out = gst_buffer_create_sub (buffer, offset * bpf, length * bpf);
 
   GST_BUFFER_OFFSET (out) = new_start;
   GST_BUFFER_OFFSET_END (out) = new_stop;
-  GST_BUFFER_TIMESTAMP (out) = new_start * wildmidi->time_per_frame;
-  GST_BUFFER_DURATION (out) = (new_stop - new_start) * wildmidi->time_per_frame;
+  GST_BUFFER_TIMESTAMP (out) =
+      gst_util_uint64_scale_int (new_start, GST_SECOND, WILDMIDI_RATE);
+  GST_BUFFER_DURATION (out) =
+      gst_util_uint64_scale_int (new_stop, GST_SECOND, WILDMIDI_RATE) -
+      GST_BUFFER_TIMESTAMP (out);
 
   gst_buffer_unref (buffer);
 
@@ -574,225 +597,248 @@ gst_wildmidi_clip_buffer (GstWildmidi * wildmidi, GstBuffer * buffer)
 
 /* generate audio data and advance internal timers */
 static GstBuffer *
-gst_wildmidi_fill_buffer (GstWildmidi * wildmidi, GstBuffer * buffer)
+gst_wildmidi_get_buffer (GstWildmidi * wildmidi)
 {
   size_t bytes_read;
   gint64 samples;
+  GstBuffer *buffer;
+  GstSegment *segment;
+  guint8 *data;
+  guint size;
+  guint bpf;
 
-  bytes_read =
-      WildMidi_GetOutput (wildmidi->song, (char *) GST_BUFFER_DATA (buffer),
-      (unsigned long int) GST_BUFFER_SIZE (buffer));
+  bpf = wildmidi->bytes_per_frame;
+
+  buffer = gst_buffer_new_and_alloc (256 * bpf);
+
+  data = GST_BUFFER_DATA (buffer);
+  size = GST_BUFFER_SIZE (buffer);
+
+  GST_OBJECT_LOCK (wildmidi);
+  bytes_read = WildMidi_GetOutput (wildmidi->song, (char *) data,
+      (unsigned long int) size);
+  GST_OBJECT_UNLOCK (wildmidi);
 
   if (bytes_read == 0) {
     gst_buffer_unref (buffer);
     return NULL;
   }
 
-  GST_BUFFER_OFFSET (buffer) =
-      wildmidi->o_segment->last_stop * wildmidi->bytes_per_frame;
+  /* adjust buffer size */
+  size = GST_BUFFER_SIZE (buffer) = bytes_read;
+
+  segment = wildmidi->o_segment;
+
+  GST_BUFFER_OFFSET (buffer) = segment->last_stop;
   GST_BUFFER_TIMESTAMP (buffer) =
-      wildmidi->o_segment->last_stop * wildmidi->time_per_frame;
+      gst_util_uint64_scale_int (segment->last_stop, GST_SECOND, WILDMIDI_RATE);
 
-  if (bytes_read < GST_BUFFER_SIZE (buffer)) {
-    GstBuffer *old = buffer;
+  samples = size / bpf;
+  segment->last_stop += samples;
 
-    buffer = gst_buffer_create_sub (buffer, 0, bytes_read);
-    gst_buffer_unref (old);
-  }
+  GST_BUFFER_OFFSET_END (buffer) = segment->last_stop;
+  GST_BUFFER_DURATION (buffer) =
+      gst_util_uint64_scale_int (segment->last_stop, GST_SECOND,
+      WILDMIDI_RATE) - GST_BUFFER_TIMESTAMP (buffer);
 
-  samples = GST_BUFFER_SIZE (buffer) / wildmidi->bytes_per_frame;
+  GST_DEBUG_OBJECT (wildmidi, "buffer ts: %" GST_TIME_FORMAT ", "
+      "duration: %" GST_TIME_FORMAT " (%" G_GINT64_FORMAT " samples)",
+      GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (buffer)),
+      GST_TIME_ARGS (GST_BUFFER_DURATION (buffer)), samples);
 
-  wildmidi->o_segment->last_stop += samples;
-
-  GST_BUFFER_OFFSET_END (buffer) =
-      wildmidi->o_segment->last_stop * wildmidi->bytes_per_frame;
-  GST_BUFFER_DURATION (buffer) = samples * wildmidi->time_per_frame;
-
-  GST_DEBUG_OBJECT (wildmidi,
-      "generated buffer %" GST_TIME_FORMAT "-%" GST_TIME_FORMAT
-      " (%d samples)",
-      GST_TIME_ARGS ((guint64) GST_BUFFER_TIMESTAMP (buffer)),
-      GST_TIME_ARGS (((guint64) (GST_BUFFER_TIMESTAMP (buffer) +
-                  GST_BUFFER_DURATION (buffer)))), samples);
-
-  return buffer;
+  return gst_wildmidi_clip_buffer (wildmidi, buffer);
 }
 
-static GstBuffer *
-gst_wildmidi_get_buffer (GstWildmidi * wildmidi)
+static GstFlowReturn
+gst_wildmidi_parse_song (GstWildmidi * wildmidi)
+{
+  struct _WM_Info *info;
+  GstCaps *outcaps;
+  guint8 *data;
+  guint size;
+
+  GST_DEBUG_OBJECT (wildmidi, "Parsing song");
+
+  size = gst_adapter_available (wildmidi->adapter);
+  data = gst_adapter_take (wildmidi->adapter, size);
+
+  /* this method takes our memory block */
+  GST_OBJECT_LOCK (wildmidi);
+  wildmidi->song = WildMidi_OpenBuffer (data, size);
+
+  if (!wildmidi->song)
+    goto open_failed;
+
+  WildMidi_LoadSamples (wildmidi->song);
+
+  WildMidi_SetOption (wildmidi->song, WM_MO_LINEAR_VOLUME,
+      wildmidi->linear_volume);
+  WildMidi_SetOption (wildmidi->song, WM_MO_EXPENSIVE_INTERPOLATION,
+      wildmidi->high_quality);
+
+  info = WildMidi_GetInfo (wildmidi->song);
+  GST_OBJECT_UNLOCK (wildmidi);
+
+  wildmidi->o_len = info->approx_total_samples;
+
+  outcaps = gst_caps_copy (gst_pad_get_pad_template_caps (wildmidi->srcpad));
+  gst_pad_set_caps (wildmidi->srcpad, outcaps);
+  gst_caps_unref (outcaps);
+
+  /* we keep an internal segment in samples */
+  gst_segment_set_newsegment (wildmidi->o_segment, FALSE, 1.0,
+      GST_FORMAT_DEFAULT, 0, GST_CLOCK_TIME_NONE, 0);
+
+  gst_pad_push_event (wildmidi->srcpad,
+      gst_wildmidi_get_new_segment_event (wildmidi, GST_FORMAT_TIME));
+
+  GST_DEBUG_OBJECT (wildmidi, "Parsing song done");
+
+  return GST_FLOW_OK;
+
+  /* ERRORS */
+open_failed:
+  {
+    GST_OBJECT_UNLOCK (wildmidi);
+    GST_ELEMENT_ERROR (wildmidi, STREAM, DECODE, (NULL),
+        ("Unable to parse midi data"));
+    return GST_FLOW_ERROR;
+  }
+}
+
+static GstFlowReturn
+gst_wildmidi_do_play (GstWildmidi * wildmidi)
 {
   GstBuffer *out;
+  GstFlowReturn ret;
 
-  out =
-      gst_wildmidi_fill_buffer (wildmidi,
-      gst_wildmidi_allocate_buffer (wildmidi, 256));
+  if (!(out = gst_wildmidi_get_buffer (wildmidi)))
+    goto eos;
 
-  if (!out)
-    return NULL;
+  if (wildmidi->discont) {
+    GST_BUFFER_FLAG_SET (out, GST_BUFFER_FLAG_DISCONT);
+    wildmidi->discont = FALSE;
+  }
 
-  return gst_wildmidi_clip_buffer (wildmidi, out);
+  gst_buffer_set_caps (out, GST_PAD_CAPS (wildmidi->srcpad));
+  ret = gst_pad_push (wildmidi->srcpad, out);
+
+  return ret;
+
+  /* ERRORS */
+eos:
+  {
+    GST_LOG_OBJECT (wildmidi, "Song ended");
+    return GST_FLOW_UNEXPECTED;
+  }
+}
+
+static gboolean
+gst_wildmidi_sink_event (GstPad * pad, GstEvent * event)
+{
+  gboolean res = FALSE;
+  GstWildmidi *wildmidi = GST_WILDMIDI (gst_pad_get_parent (pad));
+
+  GST_DEBUG_OBJECT (pad, "%s event received", GST_EVENT_TYPE_NAME (event));
+
+  switch (GST_EVENT_TYPE (event)) {
+    case GST_EVENT_EOS:
+      wildmidi->state = GST_WILDMIDI_STATE_PARSE;
+      /* now start the parsing task */
+      gst_pad_start_task (wildmidi->sinkpad,
+          (GstTaskFunction) gst_wildmidi_loop, wildmidi->sinkpad);
+      /* don't forward the event */
+      gst_event_unref (event);
+      break;
+    default:
+      res = gst_pad_push_event (wildmidi->srcpad, event);
+      break;
+  }
+  return res;
+}
+
+static GstFlowReturn
+gst_wildmidi_chain (GstPad * sinkpad, GstBuffer * buffer)
+{
+  GstWildmidi *wildmidi;
+
+  wildmidi = GST_WILDMIDI (GST_PAD_PARENT (sinkpad));
+
+  /* push stuff in the adapter, we will start doing something in the sink event
+   * handler when we get EOS */
+  gst_adapter_push (wildmidi->adapter, buffer);
+
+  return GST_FLOW_OK;
 }
 
 static void
 gst_wildmidi_loop (GstPad * sinkpad)
 {
   GstWildmidi *wildmidi = GST_WILDMIDI (GST_PAD_PARENT (sinkpad));
-  GstBuffer *out;
   GstFlowReturn ret;
 
-  if (wildmidi->mididata_size == 0) {
-    if (!gst_wildmidi_get_upstream_size (wildmidi, &wildmidi->mididata_size)) {
-      GST_ELEMENT_ERROR (wildmidi, STREAM, DECODE, (NULL),
-          ("Unable to get song length"));
-      goto paused;
+  switch (wildmidi->state) {
+    case GST_WILDMIDI_STATE_LOAD:
+    {
+      GstBuffer *buffer;
+
+      GST_DEBUG_OBJECT (wildmidi, "loading song");
+
+      ret =
+          gst_pad_pull_range (wildmidi->sinkpad, wildmidi->offset, -1, &buffer);
+
+      if (ret == GST_FLOW_UNEXPECTED) {
+        GST_DEBUG_OBJECT (wildmidi, "Song loaded");
+        wildmidi->state = GST_WILDMIDI_STATE_PARSE;
+      } else if (ret != GST_FLOW_OK) {
+        GST_ELEMENT_ERROR (wildmidi, STREAM, DECODE, (NULL),
+            ("Unable to read song"));
+        goto pause;
+      } else {
+        GST_DEBUG_OBJECT (wildmidi, "pushing buffer");
+        gst_adapter_push (wildmidi->adapter, buffer);
+        wildmidi->offset += GST_BUFFER_SIZE (buffer);
+      }
+      break;
     }
-
-    if (wildmidi->mididata)
-      free (wildmidi->mididata);
-
-    wildmidi->mididata = malloc (wildmidi->mididata_size);
-    wildmidi->mididata_offset = 0;
-    return;
+    case GST_WILDMIDI_STATE_PARSE:
+      ret = gst_wildmidi_parse_song (wildmidi);
+      if (ret != GST_FLOW_OK)
+        goto pause;
+      wildmidi->state = GST_WILDMIDI_STATE_PLAY;
+      break;
+    case GST_WILDMIDI_STATE_PLAY:
+      ret = gst_wildmidi_do_play (wildmidi);
+      if (ret != GST_FLOW_OK)
+        goto pause;
+      break;
+    default:
+      break;
   }
-
-  if (wildmidi->mididata_offset < wildmidi->mididata_size) {
-    GstBuffer *buffer;
-    gint64 size;
-
-    GST_DEBUG_OBJECT (wildmidi, "loading song");
-
-    ret =
-        gst_pad_pull_range (wildmidi->sinkpad, wildmidi->mididata_offset,
-        -1, &buffer);
-    if (ret != GST_FLOW_OK) {
-      GST_ELEMENT_ERROR (wildmidi, STREAM, DECODE, (NULL),
-          ("Unable to load song"));
-      goto paused;
-    }
-
-    size = wildmidi->mididata_size - wildmidi->mididata_offset;
-    if (GST_BUFFER_SIZE (buffer) < size)
-      size = GST_BUFFER_SIZE (buffer);
-
-    memmove (wildmidi->mididata + wildmidi->mididata_offset,
-        GST_BUFFER_DATA (buffer), size);
-    gst_buffer_unref (buffer);
-
-    wildmidi->mididata_offset += size;
-    GST_DEBUG_OBJECT (wildmidi, "Song loaded");
-    return;
-  }
-
-  if (!wildmidi->song) {
-    struct _WM_Info *info;
-
-    GST_DEBUG_OBJECT (wildmidi, "Parsing song");
-
-    /* this method takes our memory block */
-    wildmidi->song =
-        WildMidi_OpenBuffer ((unsigned char *) wildmidi->mididata,
-        wildmidi->mididata_size);
-    wildmidi->mididata_size = 0;
-    wildmidi->mididata = NULL;
-
-    if (!wildmidi->song) {
-      GST_ELEMENT_ERROR (wildmidi, STREAM, DECODE, (NULL),
-          ("Unable to parse midi"));
-      goto paused;
-    }
-
-    WildMidi_LoadSamples (wildmidi->song);
-
-    WildMidi_SetOption (wildmidi->song, WM_MO_LINEAR_VOLUME,
-        wildmidi->linear_volume);
-    WildMidi_SetOption (wildmidi->song, WM_MO_EXPENSIVE_INTERPOLATION,
-        wildmidi->high_quality);
-
-    info = WildMidi_GetInfo (wildmidi->song);
-    wildmidi->o_len = info->approx_total_samples;
-
-    gst_segment_set_newsegment (wildmidi->o_segment, FALSE, 1.0,
-        GST_FORMAT_DEFAULT, 0, GST_CLOCK_TIME_NONE, 0);
-
-    gst_pad_push_event (wildmidi->srcpad,
-        gst_wildmidi_get_new_segment_event (wildmidi, GST_FORMAT_TIME, FALSE));
-
-    GST_DEBUG_OBJECT (wildmidi, "Parsing song done");
-    return;
-  }
-
-  if (wildmidi->o_segment_changed) {
-    GST_DEBUG_OBJECT (wildmidi, "segment changed");
-
-    GstSegment *segment = gst_wildmidi_get_segment (wildmidi, GST_FORMAT_TIME,
-        !wildmidi->o_new_segment);
-
-    GST_LOG_OBJECT (wildmidi,
-        "sending newsegment from %" GST_TIME_FORMAT "-%" GST_TIME_FORMAT
-        ", pos=%" GST_TIME_FORMAT, GST_TIME_ARGS ((guint64) segment->start),
-        GST_TIME_ARGS ((guint64) segment->stop),
-        GST_TIME_ARGS ((guint64) segment->time));
-
-    if (wildmidi->o_segment->flags & GST_SEEK_FLAG_SEGMENT) {
-      gst_element_post_message (GST_ELEMENT (wildmidi),
-          gst_message_new_segment_start (GST_OBJECT (wildmidi),
-              segment->format, segment->start));
-    }
-
-    gst_segment_free (segment);
-    wildmidi->o_segment_changed = FALSE;
-    return;
-  }
-
-  if (wildmidi->o_seek) {
-    unsigned long int sample;
-
-    /* perform a seek internally */
-    sample = wildmidi->o_segment->time;
-
-    if (wildmidi->accurate_seek) {
-      WildMidi_SampledSeek (wildmidi->song, &sample);
-    } else {
-      WildMidi_FastSeek (wildmidi->song, &sample);
-    }
-
-    wildmidi->o_segment->last_stop = wildmidi->o_segment->time = sample;
-  }
-
-  out = gst_wildmidi_get_buffer (wildmidi);
-  if (!out) {
-    GST_LOG_OBJECT (wildmidi, "Song ended, generating eos");
-    gst_pad_push_event (wildmidi->srcpad, gst_event_new_eos ());
-    wildmidi->o_seek = FALSE;
-    goto paused;
-  }
-
-  if (wildmidi->o_seek) {
-    GST_BUFFER_FLAG_SET (out, GST_BUFFER_FLAG_DISCONT);
-    wildmidi->o_seek = FALSE;
-  }
-
-  gst_buffer_set_caps (out, wildmidi->out_caps);
-  ret = gst_pad_push (wildmidi->srcpad, out);
-
-  if (GST_FLOW_IS_FATAL (ret) || ret == GST_FLOW_NOT_LINKED)
-    goto error;
-
   return;
 
-paused:
+pause:
   {
-    GST_DEBUG_OBJECT (wildmidi, "pausing task");
-    gst_pad_pause_task (wildmidi->sinkpad);
-    return;
-  }
-error:
-  {
-    GST_ELEMENT_ERROR (wildmidi, STREAM, FAILED,
-        ("Internal data stream error"),
-        ("Streaming stopped, reason %s", gst_flow_get_name (ret)));
-    gst_pad_push_event (wildmidi->srcpad, gst_event_new_eos ());
-    goto paused;
+    const gchar *reason = gst_flow_get_name (ret);
+    GstEvent *event;
+
+    GST_DEBUG_OBJECT (wildmidi, "pausing task, reason %s", reason);
+    gst_pad_pause_task (sinkpad);
+    if (GST_FLOW_IS_FATAL (ret) || ret == GST_FLOW_NOT_LINKED) {
+      if (ret == GST_FLOW_UNEXPECTED) {
+        /* perform EOS logic */
+        event = gst_event_new_eos ();
+        gst_pad_push_event (wildmidi->srcpad, event);
+      } else {
+        event = gst_event_new_eos ();
+        /* for fatal errors we post an error message, post the error
+         * first so the app knows about the error first. */
+        GST_ELEMENT_ERROR (wildmidi, STREAM, FAILED,
+            ("Internal data flow error."),
+            ("streaming task paused, reason %s (%d)", reason, ret));
+        gst_pad_push_event (wildmidi->srcpad, event);
+      }
+    }
   }
 }
 
@@ -802,19 +848,13 @@ gst_wildmidi_change_state (GstElement * element, GstStateChange transition)
   GstStateChangeReturn ret = GST_STATE_CHANGE_SUCCESS;
   GstWildmidi *wildmidi = GST_WILDMIDI (element);
 
-  if (!wildmidi->initialized) {
-    GST_WARNING ("WildMidi renderer is not initialized");
-    return GST_STATE_CHANGE_FAILURE;
-  }
-
   switch (transition) {
     case GST_STATE_CHANGE_NULL_TO_READY:
-      wildmidi->out_caps =
-          gst_caps_copy (gst_pad_get_pad_template_caps (wildmidi->srcpad));
-      wildmidi->mididata = NULL;
       break;
     case GST_STATE_CHANGE_READY_TO_PAUSED:
-      wildmidi->mididata_size = 0;
+      wildmidi->offset = 0;
+      wildmidi->state = GST_WILDMIDI_STATE_LOAD;
+      wildmidi->discont = FALSE;
       break;
     case GST_STATE_CHANGE_PAUSED_TO_PLAYING:
       break;
@@ -828,15 +868,14 @@ gst_wildmidi_change_state (GstElement * element, GstStateChange transition)
     case GST_STATE_CHANGE_PLAYING_TO_PAUSED:
       break;
     case GST_STATE_CHANGE_PAUSED_TO_READY:
+      GST_OBJECT_LOCK (wildmidi);
       if (wildmidi->song)
         WildMidi_Close (wildmidi->song);
       wildmidi->song = NULL;
-      if (wildmidi->mididata)
-        free (wildmidi->mididata);
-      wildmidi->mididata = NULL;
+      GST_OBJECT_UNLOCK (wildmidi);
+      gst_adapter_clear (wildmidi->adapter);
       break;
     case GST_STATE_CHANGE_READY_TO_NULL:
-      gst_caps_unref (wildmidi->out_caps);
       break;
     default:
       break;
@@ -857,16 +896,20 @@ gst_wildmidi_set_property (GObject * object, guint prop_id,
 
   switch (prop_id) {
     case ARG_LINEAR_VOLUME:
+      GST_OBJECT_LOCK (object);
       wildmidi->linear_volume = g_value_get_boolean (value);
       if (wildmidi->song)
         WildMidi_SetOption (wildmidi->song, WM_MO_LINEAR_VOLUME,
             wildmidi->linear_volume);
+      GST_OBJECT_UNLOCK (object);
       break;
     case ARG_HIGH_QUALITY:
+      GST_OBJECT_LOCK (object);
       wildmidi->high_quality = g_value_get_boolean (value);
       if (wildmidi->song)
         WildMidi_SetOption (wildmidi->song, WM_MO_EXPENSIVE_INTERPOLATION,
             wildmidi->high_quality);
+      GST_OBJECT_UNLOCK (object);
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -886,10 +929,14 @@ gst_wildmidi_get_property (GObject * object, guint prop_id,
 
   switch (prop_id) {
     case ARG_LINEAR_VOLUME:
+      GST_OBJECT_LOCK (object);
       g_value_set_boolean (value, wildmidi->linear_volume);
+      GST_OBJECT_UNLOCK (object);
       break;
     case ARG_HIGH_QUALITY:
+      GST_OBJECT_LOCK (object);
       g_value_set_boolean (value, wildmidi->high_quality);
+      GST_OBJECT_UNLOCK (object);
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -903,6 +950,11 @@ plugin_init (GstPlugin * plugin)
   GST_DEBUG_CATEGORY_INIT (gst_wildmidi_debug, "wildmidi",
       0, "Wildmidi plugin");
 
+  if (!wildmidi_open_config ()) {
+    GST_WARNING ("Can't initialize wildmidi");
+    return FALSE;
+  }
+
   return gst_element_register (plugin, "wildmidi",
       GST_RANK_SECONDARY, GST_TYPE_WILDMIDI);
 }
