@@ -36,13 +36,6 @@
 #include <gst/video/video.h>
 #include "gstxvidenc.h"
 
-/* elementfactory information */
-static const GstElementDetails gst_xvidenc_details =
-GST_ELEMENT_DETAILS ("XviD video encoder",
-    "Codec/Encoder/Video",
-    "XviD encoder based on xvidcore",
-    "Ronald Bultje <rbultje@ronald.bitfreak.net>");
-
 static GstStaticPadTemplate sink_template = GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
     GST_PAD_ALWAYS,
@@ -59,6 +52,11 @@ static GstStaticPadTemplate src_template = GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
     GST_PAD_ALWAYS,
     GST_STATIC_CAPS ("video/x-xvid, "
+        "width = (int) [ 0, MAX ], "
+        "height = (int) [ 0, MAX ], " "framerate = (fraction) [ 0/1, MAX ]; "
+        "video/mpeg, "
+        "mpegversion = (int) 4, "
+        "systemstream = (boolean) FALSE, "
         "width = (int) [ 0, MAX ], "
         "height = (int) [ 0, MAX ], " "framerate = (fraction) [ 0/1, MAX ]")
     );
@@ -199,9 +197,17 @@ gst_xvidenc_get_type (void)
       0,
       (GInstanceInitFunc) gst_xvidenc_init,
     };
+    const GInterfaceInfo preset_interface_info = {
+      NULL,                     /* interface_init */
+      NULL,                     /* interface_finalize */
+      NULL                      /* interface_data */
+    };
 
     xvidenc_type = g_type_register_static (GST_TYPE_ELEMENT,
         "GstXvidEnc", &xvidenc_info, 0);
+
+    g_type_add_interface_static (xvidenc_type, GST_TYPE_PRESET,
+        &preset_interface_info);
   }
   return xvidenc_type;
 }
@@ -215,7 +221,10 @@ gst_xvidenc_base_init (GstXvidEncClass * klass)
       gst_static_pad_template_get (&sink_template));
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&src_template));
-  gst_element_class_set_details (element_class, &gst_xvidenc_details);
+  gst_element_class_set_details_simple (element_class, "XviD video encoder",
+      "Codec/Encoder/Video",
+      "XviD encoder based on xvidcore",
+      "Ronald Bultje <rbultje@ronald.bitfreak.net>");
 }
 
 /* add property pspec to klass using the counter count,
@@ -723,31 +732,37 @@ gst_xvidenc_setcaps (GstPad * pad, GstCaps * vscaps)
   xvidenc->xframe_cache = NULL;
 
   if (gst_xvidenc_setup (xvidenc)) {
-    GstPadLinkReturn ret;
-    GstCaps *new_caps;
-    GstPad *peer;
+    gboolean ret = FALSE;
+    GstCaps *new_caps = NULL, *allowed_caps;
 
-    new_caps = gst_caps_new_simple ("video/x-xvid",
+    /* please downstream with preferred caps */
+    allowed_caps = gst_pad_get_allowed_caps (xvidenc->srcpad);
+    GST_DEBUG_OBJECT (xvidenc, "allowed caps: %" GST_PTR_FORMAT, allowed_caps);
+
+    if (allowed_caps && !gst_caps_is_empty (allowed_caps)) {
+      new_caps = gst_caps_copy_nth (allowed_caps, 0);
+    } else {
+      new_caps = gst_caps_new_simple ("video/x-xvid", NULL);
+    }
+    if (allowed_caps)
+      gst_caps_unref (allowed_caps);
+
+    gst_caps_set_simple (new_caps,
         "width", G_TYPE_INT, w, "height", G_TYPE_INT, h,
         "framerate", GST_TYPE_FRACTION, xvidenc->fbase, xvidenc->fincr,
         "pixel-aspect-ratio", GST_TYPE_FRACTION,
         xvidenc->par_width, xvidenc->par_height, NULL);
+    /* just to be sure */
+    gst_pad_fixate_caps (xvidenc->srcpad, new_caps);
+
     /* src pad should accept anyway */
     ret = gst_pad_set_caps (xvidenc->srcpad, new_caps);
-    if (!ret)
-      goto exit;
-    /* will peer accept */
-    peer = gst_pad_get_peer (xvidenc->srcpad);
-    if (peer)
-      ret &= gst_pad_accept_caps (peer, new_caps);
-    gst_object_unref (peer);
+    gst_caps_unref (new_caps);
 
-  exit:
     if (!ret && xvidenc->handle) {
       xvid_encore (xvidenc->handle, XVID_ENC_DESTROY, NULL, NULL);
       xvidenc->handle = NULL;
     }
-    gst_caps_unref (new_caps);
     return ret;
 
   } else                        /* setup did not work out */
