@@ -28,10 +28,15 @@
 #include "photography.h"
 
 /**
- * SECTION:photography
- * @short_description: Interface for elements having digital imaging controls
+ * SECTION:gstphotography
+ * @short_description: Interface for digital image capture elements
  *
- * The interface allows access to some common digital imaging controls
+ * The interface allows access to some common digital image capture parameters.
+ *
+ * <note>
+ *   The GstPhotography interface is unstable API and may change in future.
+ *   One can define GST_USE_UNSTABLE_API to acknowledge and avoid this warning.
+ * </note>
  */
 
 static void gst_photography_iface_base_init (GstPhotographyInterface * iface);
@@ -77,6 +82,8 @@ gst_photography_iface_base_init (GstPhotographyInterface * iface)
   iface->get_scene_mode = NULL;
   iface->get_flash_mode = NULL;
   iface->get_zoom = NULL;
+  iface->get_flicker_mode = NULL;
+  iface->get_focus_mode = NULL;
 
   iface->set_ev_compensation = NULL;
   iface->set_iso_speed = NULL;
@@ -87,6 +94,8 @@ gst_photography_iface_base_init (GstPhotographyInterface * iface)
   iface->set_scene_mode = NULL;
   iface->set_flash_mode = NULL;
   iface->set_zoom = NULL;
+  iface->set_flicker_mode = NULL;
+  iface->set_focus_mode = NULL;
 
   iface->get_capabilities = NULL;
   iface->prepare_for_capture = NULL;
@@ -301,6 +310,46 @@ GST_PHOTOGRAPHY_FUNC_TEMPLATE (flash_mode, GstFlashMode);
  */
 GST_PHOTOGRAPHY_FUNC_TEMPLATE (zoom, gfloat);
 
+/**
+ * gst_photography_set_flicker_mode:
+ * @photo: #GstPhotography interface of a #GstElement
+ * @flicker_mode: flicker mode value to set
+ *
+ * Set the flicker mode value for the #GstElement.
+ *
+ * Returns: %TRUE if setting the value succeeded, %FALSE otherwise
+ */
+/**
+ * gst_photography_get_flicker_mode:
+ * @photo: #GstPhotography interface of a #GstElement
+ * @flicker_mode: flicker mode value to get
+ *
+ * Get the flicker mode value for the #GstElement
+ *
+ * Returns: %TRUE if getting the value succeeded, %FALSE otherwise
+ */
+GST_PHOTOGRAPHY_FUNC_TEMPLATE (flicker_mode, GstFlickerReductionMode);
+
+/**
+ * gst_photography_set_focus_mode:
+ * @photo: #GstPhotography interface of a #GstElement
+ * @focus_mode: focus mode value to set
+ *
+ * Set the focus mode value for the #GstElement.
+ *
+ * Returns: %TRUE if setting the value succeeded, %FALSE otherwise
+ */
+/**
+ * gst_photography_get_focus_mode:
+ * @photo: #GstPhotography interface of a #GstElement
+ * @focus_mode: focus_mode value to get
+ *
+ * Get the focus mode value for the #GstElement
+ *
+ * Returns: %TRUE if getting the value succeeded, %FALSE otherwise
+ */
+GST_PHOTOGRAPHY_FUNC_TEMPLATE (focus_mode, GstFocusMode);
+
 /**
  * gst_photography_get_capabilities:
  * @photo: #GstPhotography interface of a #GstElement
@@ -435,7 +484,8 @@ gst_photography_iface_class_init (gpointer g_class)
           "White balance mode property",
           "White balance affects the color temperature of the photo",
           GST_TYPE_WHITE_BALANCE_MODE,
-          GST_PHOTOGRAPHY_WB_MODE_AUTO, G_PARAM_READWRITE));
+          GST_PHOTOGRAPHY_WB_MODE_AUTO,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   /* Colour tone */
   g_object_interface_install_property (g_class,
@@ -443,7 +493,8 @@ gst_photography_iface_class_init (gpointer g_class)
           "Colour tone mode property",
           "Colour tone setting changes colour shading in the photo",
           GST_TYPE_COLOUR_TONE_MODE,
-          GST_PHOTOGRAPHY_COLOUR_TONE_MODE_NORMAL, G_PARAM_READWRITE));
+          GST_PHOTOGRAPHY_COLOUR_TONE_MODE_NORMAL,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   /* Scene mode */
   g_object_interface_install_property (g_class,
@@ -451,7 +502,8 @@ gst_photography_iface_class_init (gpointer g_class)
           "Scene mode property",
           "Scene mode works as a preset for different photo shooting mode settings",
           GST_TYPE_SCENE_MODE,
-          GST_PHOTOGRAPHY_SCENE_MODE_AUTO, G_PARAM_READWRITE));
+          GST_PHOTOGRAPHY_SCENE_MODE_AUTO,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   /* Flash mode */
   g_object_interface_install_property (g_class,
@@ -459,40 +511,66 @@ gst_photography_iface_class_init (gpointer g_class)
           "Flash mode property",
           "Flash mode defines how the flash light should be used",
           GST_TYPE_FLASH_MODE,
-          GST_PHOTOGRAPHY_FLASH_MODE_AUTO, G_PARAM_READWRITE));
+          GST_PHOTOGRAPHY_FLASH_MODE_AUTO,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+
+  /* Flicker reduction mode */
+  g_object_interface_install_property (g_class,
+      g_param_spec_enum (GST_PHOTOGRAPHY_PROP_FLICKER_MODE,
+          "Flicker reduction mode property",
+          "Flicker reduction mode defines a line frequency for flickering prevention",
+          GST_TYPE_FLICKER_REDUCTION_MODE,
+          GST_PHOTOGRAPHY_FLICKER_REDUCTION_OFF,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+
+  /* Focus mode */
+  g_object_interface_install_property (g_class,
+      g_param_spec_enum (GST_PHOTOGRAPHY_PROP_FOCUS_MODE,
+          "Focus mode property",
+          "Focus mode defines the range of focal lengths to use in autofocus search",
+          GST_TYPE_FOCUS_MODE,
+          GST_PHOTOGRAPHY_FOCUS_MODE_AUTO,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   /* Capabilities */
   g_object_interface_install_property (g_class,
       g_param_spec_ulong (GST_PHOTOGRAPHY_PROP_CAPABILITIES,
           "Photo capabilities bitmask",
           "Tells the photo capabilities of the device",
-          0, G_MAXULONG, 0, G_PARAM_READABLE));
+          0, G_MAXULONG, 0, G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
   /* EV_compensation */
   g_object_interface_install_property (g_class,
       g_param_spec_float (GST_PHOTOGRAPHY_PROP_EV_COMP,
           "EV compensation property",
           "EV compensation affects the brightness of the image",
-          -2.5, 2.5, 0, G_PARAM_READWRITE));
+          -2.5, 2.5, 0, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   /* ISO value */
   g_object_interface_install_property (g_class,
       g_param_spec_uint (GST_PHOTOGRAPHY_PROP_ISO_SPEED,
           "ISO speed property",
           "ISO speed defines the light sensitivity (0 = auto)",
-          0, 6400, 0, G_PARAM_READWRITE));
+          0, 6400, 0, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   /* Aperture */
   g_object_interface_install_property (g_class,
       g_param_spec_uint (GST_PHOTOGRAPHY_PROP_APERTURE,
           "Aperture property",
           "Aperture defines the size of lens opening  (0 = auto)",
-          0, G_MAXUINT8, 0, G_PARAM_READWRITE));
+          0, G_MAXUINT8, 0, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   /* Exposure */
   g_object_interface_install_property (g_class,
       g_param_spec_uint (GST_PHOTOGRAPHY_PROP_EXPOSURE,
           "Exposure time in milliseconds",
           "Exposure time defines how long the shutter will stay open (0 = auto)",
-          0, G_MAXUINT32, 0, G_PARAM_READWRITE));
+          0, G_MAXUINT32, 0, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+
+  /* Image capture caps */
+  g_object_interface_install_property (g_class,
+      g_param_spec_boxed (GST_PHOTOGRAPHY_PROP_IMAGE_CAPTURE_SUPPORTED_CAPS,
+          "Image capture supported caps",
+          "Caps describing supported image capture formats", GST_TYPE_CAPS,
+          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 }
