@@ -127,12 +127,6 @@ typedef GstAsfExtContDescData GstAsfMetadataObjData;
 #define DEFAULT_PADDING 0
 #define DEFAULT_IS_LIVE FALSE
 
-static const GstElementDetails gst_asf_mux_details =
-GST_ELEMENT_DETAILS ("ASF muxer",
-    "Codec/Muxer",
-    "Muxes audio and video into an ASF stream",
-    "Thiago Santos <thiagoss@embedded.ufcg.edu.br>");
-
 static GstStaticPadTemplate src_factory = GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
     GST_PAD_ALWAYS,
@@ -249,7 +243,10 @@ gst_asf_mux_base_init (gpointer g_class)
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&video_sink_factory));
 
-  gst_element_class_set_details (element_class, &gst_asf_mux_details);
+  gst_element_class_set_details_simple (element_class, "ASF muxer",
+      "Codec/Muxer",
+      "Muxes audio and video into an ASF stream",
+      "Thiago Santos <thiagoss@embedded.ufcg.edu.br>");
 
   GST_DEBUG_CATEGORY_INIT (asfmux_debug, "asfmux", 0, "Muxer for ASF streams");
 }
@@ -306,8 +303,8 @@ gst_asf_mux_class_init (GstAsfMuxClass * klass)
           DEFAULT_PADDING, G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
   g_object_class_install_property (gobject_class, PROP_IS_LIVE,
       g_param_spec_boolean ("is-live", "Is Live",
-          "If this stream should be threated as a live, meaning that it "
-          "doesn't need indexes nor late update of headers.",
+          "Whether this stream should be treated as a live stream, meaning "
+          "that it doesn't need an index or header updates when done.",
           DEFAULT_IS_LIVE, G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
 
   gstelement_class->request_new_pad =
@@ -1223,7 +1220,7 @@ gst_asf_mux_write_data_object (GstAsfMux * asfmux, guint8 ** buf)
   *buf += ASF_DATA_OBJECT_SIZE;
 }
 
-guint
+static guint
 gst_asf_mux_find_payload_parsing_info_size (GstAsfMux * asfmux)
 {
   /* Minimum payload parsing information size is 8 bytes */
@@ -1270,7 +1267,7 @@ gst_asf_mux_start_file (GstAsfMux * asfmux)
   gst_pad_push_event (asfmux->srcpad,
       gst_event_new_new_segment (FALSE, 1.0, GST_FORMAT_BYTES, 0, -1, 0));
 
-  asfmux->file_id = gst_asf_generate_file_id ();
+  gst_asf_generate_file_id (&asfmux->file_id);
 
   /* Get the metadata for content description object.
    * We store our own taglist because it might get changed from now
