@@ -52,6 +52,7 @@
 
 #include "camerabinimage.h"
 #include "camerabingeneral.h"
+#include "gstcamerabin-enum.h"
 
 #include "string.h"
 
@@ -60,6 +61,7 @@
 #define DEFAULT_SINK "filesink"
 #define DEFAULT_ENC "jpegenc"
 #define DEFAULT_META_MUX "metadatamux"
+#define DEFAULT_FLAGS GST_CAMERABIN_FLAG_IMAGE_COLOR_CONVERSION
 
 enum
 {
@@ -83,13 +85,6 @@ static void gst_camerabin_image_get_property (GObject * object, guint prop_id,
 
 GST_BOILERPLATE (GstCameraBinImage, gst_camerabin_image, GstBin, GST_TYPE_BIN);
 
-static const GstElementDetails gst_camerabin_image_details =
-GST_ELEMENT_DETAILS ("Image capture bin for camerabin",
-    "Bin/Image",
-    "Process and store image data",
-    "Edgard Lima <edgard.lima@indt.org.br>\n"
-    "Nokia Corporation <multimedia@maemo.org>");
-
 static GstStaticPadTemplate sink_template = GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
     GST_PAD_ALWAYS,
@@ -109,7 +104,11 @@ gst_camerabin_image_base_init (gpointer klass)
       gst_static_pad_template_get (&sink_template));
   gst_element_class_add_pad_template (eklass,
       gst_static_pad_template_get (&src_template));
-  gst_element_class_set_details (eklass, &gst_camerabin_image_details);
+  gst_element_class_set_details_simple (eklass,
+      "Image capture bin for camerabin", "Bin/Image",
+      "Process and store image data",
+      "Edgard Lima <edgard.lima@indt.org.br>, "
+      "Nokia Corporation <multimedia@maemo.org>");
 }
 
 static void
@@ -148,7 +147,7 @@ gst_camerabin_image_init (GstCameraBinImage * img,
 
   img->post = NULL;
   img->enc = NULL;
-  img->user_enc = NULL;
+  img->app_enc = NULL;
   img->meta_mux = NULL;
   img->sink = NULL;
 
@@ -157,6 +156,7 @@ gst_camerabin_image_init (GstCameraBinImage * img,
   gst_element_add_pad (GST_ELEMENT (img), img->sinkpad);
 
   img->elements_created = FALSE;
+  img->flags = DEFAULT_FLAGS;
 }
 
 static void
@@ -167,9 +167,9 @@ gst_camerabin_image_dispose (GstCameraBinImage * img)
   g_string_free (img->filename, TRUE);
   img->filename = NULL;
 
-  if (img->user_enc) {
-    gst_object_unref (img->user_enc);
-    img->user_enc = NULL;
+  if (img->app_enc) {
+    gst_object_unref (img->app_enc);
+    img->app_enc = NULL;
   }
 
   if (img->post) {
@@ -425,21 +425,19 @@ gst_camerabin_image_create_elements (GstCameraBinImage * img)
     img_sinkpad = gst_element_get_static_pad (img->post, "sink");
   }
 
-  /* Add colorspace converter */
-  if (!(csp =
-          gst_camerabin_create_and_add_element (imgbin, "ffmpegcolorspace"))) {
-    goto done;
+  if (img->flags & GST_CAMERABIN_FLAG_IMAGE_COLOR_CONVERSION) {
+    /* Add colorspace converter */
+    if (!(csp =
+            gst_camerabin_create_and_add_element (imgbin,
+                "ffmpegcolorspace"))) {
+      goto done;
+    }
+    if (!img_sinkpad)
+      img_sinkpad = gst_element_get_static_pad (csp, "sink");
   }
 
-  /* Set up sink ghost pad for img bin */
-  if (!img_sinkpad) {
-    img_sinkpad = gst_element_get_static_pad (csp, "sink");
-  }
-  gst_ghost_pad_set_target (GST_GHOST_PAD (img->sinkpad), img_sinkpad);
-
-  /* Create image encoder */
-  if (img->user_enc) {
-    img->enc = img->user_enc;
+  if (img->app_enc) {
+    img->enc = img->app_enc;
     if (!gst_camerabin_add_element (imgbin, img->enc)) {
       goto done;
     }
@@ -471,6 +469,12 @@ gst_camerabin_image_create_elements (GstCameraBinImage * img)
   g_object_set (G_OBJECT (img->sink), "location", img->filename->str, "async", FALSE, "buffer-mode", 2, /* non buffered io */
       NULL);
 
+  /* Set up sink ghost pad for image bin */
+  if (!img_sinkpad) {
+    img_sinkpad = gst_element_get_static_pad (img->enc, "sink");
+  }
+  gst_ghost_pad_set_target (GST_GHOST_PAD (img->sinkpad), img_sinkpad);
+
   ret = TRUE;
 
 done:
@@ -514,12 +518,12 @@ void
 gst_camerabin_image_set_encoder (GstCameraBinImage * img, GstElement * encoder)
 {
   GST_DEBUG ("setting image encoder %" GST_PTR_FORMAT, encoder);
-  if (img->user_enc)
-    gst_object_unref (img->user_enc);
+  if (img->app_enc)
+    gst_object_unref (img->app_enc);
   if (encoder)
     gst_object_ref (encoder);
 
-  img->user_enc = encoder;
+  img->app_enc = encoder;
 }
 
 void
@@ -535,13 +539,20 @@ gst_camerabin_image_set_postproc (GstCameraBinImage * img,
   img->post = postproc;
 }
 
+void
+gst_camerabin_image_set_flags (GstCameraBinImage * img, GstCameraBinFlags flags)
+{
+  GST_DEBUG_OBJECT (img, "setting image flags: %d", flags);
+  img->flags = flags;
+}
+
 GstElement *
 gst_camerabin_image_get_encoder (GstCameraBinImage * img)
 {
   GstElement *enc;
 
-  if (img->user_enc) {
-    enc = img->user_enc;
+  if (img->app_enc) {
+    enc = img->app_enc;
   } else {
     enc = img->enc;
   }
