@@ -78,13 +78,6 @@ static gboolean gst_dccp_server_sink_stop (GstBaseSink * bsink);
 
 GST_DEBUG_CATEGORY_STATIC (dccpserversink_debug);
 
-static const GstElementDetails gst_dccp_server_sink_details =
-GST_ELEMENT_DETAILS ("DCCP server sink",
-    "Sink/Network",
-    "Send data as a server over the network via DCCP",
-    "E-Phone Team at Federal University of Campina Grande <leandroal@gmail.com>");
-
-
 static GstStaticPadTemplate sinktemplate = GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
     GST_PAD_ALWAYS,
@@ -337,7 +330,10 @@ gst_dccp_server_sink_base_init (gpointer g_class)
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&sinktemplate));
 
-  gst_element_class_set_details (element_class, &gst_dccp_server_sink_details);
+  gst_element_class_set_details_simple (element_class, "DCCP server sink",
+      "Sink/Network",
+      "Send data as a server over the network via DCCP",
+      "E-Phone Team at Federal University of Campina Grande <leandroal@gmail.com>");
 }
 
 /*
