@@ -26,6 +26,7 @@
 #include "gstfrei0rsrc.h"
 #include "gstfrei0rmixer.h"
 
+#include <gst/controller/gstcontroller.h>
 #include <string.h>
 
 GST_DEBUG_CATEGORY (frei0r_debug);
@@ -286,7 +287,7 @@ gst_frei0r_get_property (f0r_instance_t * instance, GstFrei0rFuncTable * ftable,
       if (instance)
         ftable->get_param_value (instance, &d, prop->prop_idx);
       else
-        d = property_cache[prop->prop_idx].data.b ? 1.0 : 0.0;
+        d = property_cache[prop->prop_idx].data.b;
 
       g_value_set_boolean (value, (d < 0.5) ? FALSE : TRUE);
       break;
@@ -386,7 +387,7 @@ gst_frei0r_set_property (f0r_instance_t * instance, GstFrei0rFuncTable * ftable,
 
       if (instance)
         ftable->set_param_value (instance, &d, prop->prop_idx);
-      property_cache[prop->prop_idx].data.b = b;
+      property_cache[prop->prop_idx].data.b = d;
       break;
     }
     case F0R_PARAM_DOUBLE:{
@@ -612,6 +613,8 @@ plugin_init (GstPlugin * plugin)
 
   GST_DEBUG_CATEGORY_INIT (frei0r_debug, "frei0r", 0, "frei0r");
 
+  gst_controller_init (NULL, NULL);
+
   gst_plugin_add_dependency_simple (plugin,
       "HOME/.frei0r-1/lib",
       "/usr/lib/frei0r-1:/usr/local/lib/frei0r-1",
