@@ -55,14 +55,6 @@ GST_DEBUG_CATEGORY_STATIC (rfbsrc_debug);
 GST_DEBUG_CATEGORY (rfbdecoder_debug);
 #define GST_CAT_DEFAULT rfbsrc_debug
 
-static const GstElementDetails gst_rfb_src_details =
-GST_ELEMENT_DETAILS ("Rfb source",
-    "Source/Video",
-    "Creates a rfb video stream",
-    "David A. Schleef <ds@schleef.org>, "
-    "Andre Moreira Magalhaes <andre.magalhaes@indt.org.br>, "
-    "Thijs Vermeir <thijsvermeir@gmail.com>");
-
 static GstStaticPadTemplate gst_rfb_src_template =
 GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
@@ -78,8 +70,6 @@ GST_STATIC_PAD_TEMPLATE ("src",
         "height = (int) [ 16, 4096 ], " "framerate = (fraction) 0/1")
     );
 
-static void gst_rfb_src_base_init (gpointer g_class);
-static void gst_rfb_src_class_init (GstRfbSrcClass * klass);
 static void gst_rfb_src_dispose (GObject * object);
 static void gst_rfb_src_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec);
@@ -102,7 +92,12 @@ gst_rfb_src_base_init (gpointer g_class)
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&gst_rfb_src_template));
 
-  gst_element_class_set_details (element_class, &gst_rfb_src_details);
+  gst_element_class_set_details_simple (element_class, "Rfb source",
+      "Source/Video",
+      "Creates a rfb video stream",
+      "David A. Schleef <ds@schleef.org>, "
+      "Andre Moreira Magalhaes <andre.magalhaes@indt.org.br>, "
+      "Thijs Vermeir <thijsvermeir@gmail.com>");
 }
 
 static void
