@@ -77,33 +77,6 @@ typedef enum
 GST_DEBUG_CATEGORY_STATIC (gstflupsdemux_debug);
 #define GST_CAT_DEFAULT (gstflupsdemux_debug)
 
-/* elementfactory information */
-static GstElementDetails flups_demux_details = {
-  "The Fluendo MPEG Program Stream Demuxer",
-  "Codec/Demuxer",
-  "Demultiplexes MPEG Program Streams",
-  "Wim Taymans <wim@fluendo.com>"
-};
-
-#define GST_TYPE_FLUPS_DEMUX_SYNC          (gst_flups_demux_sync_get_type ())
-GType
-gst_flups_demux_sync_get_type (void)
-{
-  static GType sync_type = 0;
-  static GEnumValue sync_types[] = {
-    {GST_FLUPS_DEMUX_SYNC_AUTO, "Auto-select method to synchronize streams",
-        "auto"},
-    {GST_FLUPS_DEMUX_SYNC_SCR, "Use SCR to synchronize streams", "scr"},
-    {GST_FLUPS_DEMUX_SYNC_DTS, "Use DTS to synchronize streams", "dts"},
-    {0, NULL, NULL}
-  };
-
-  if (sync_type == 0)
-    sync_type = g_enum_register_static ("GstPSDemuxSync", sync_types);
-
-  return sync_type;
-}
-
 /* MPEG2Demux signals and args */
 enum
 {
@@ -241,7 +214,9 @@ gst_flups_demux_base_init (GstFluPSDemuxClass * klass)
   gst_element_class_add_pad_template (element_class, klass->private_template);
   gst_element_class_add_pad_template (element_class, klass->sink_template);
 
-  gst_element_class_set_details (element_class, &flups_demux_details);
+  gst_element_class_set_details_simple (element_class,
+      "The Fluendo MPEG Program Stream Demuxer", "Codec/Demuxer",
+      "Demultiplexes MPEG Program Streams", "Wim Taymans <wim@fluendo.com>");
 }
 
 static void
@@ -1497,8 +1472,8 @@ gst_flups_demux_parse_pack_start (GstFluPSDemux * demux)
   /* skip start code */
   data += 4;
 
-  scr1 = GUINT32_FROM_BE (*(guint32 *) data);
-  scr2 = GUINT32_FROM_BE (*(guint32 *) (data + 4));
+  scr1 = GST_READ_UINT32_BE (data);
+  scr2 = GST_READ_UINT32_BE (data + 4);
 
   /* fixed length to begin with, start code and two scr values */
   length = 8 + 4;
@@ -1544,7 +1519,7 @@ gst_flups_demux_parse_pack_start (GstFluPSDemux * demux)
        to DTS/PTS, that also implies 1 tick rounding error */
     data += 6;
     /* PMR:22 ! :2==11 ! reserved:5 ! stuffing_len:3 */
-    next32 = (GUINT32_FROM_BE ((*(guint32 *) data)));
+    next32 = GST_READ_UINT32_BE (data);
     if (G_UNLIKELY ((next32 & 0x00000300) != 0x00000300))
       goto lost_sync;
 
@@ -2227,8 +2202,8 @@ gst_flups_demux_scan_ts (GstFluPSDemux * demux, const guint8 * data,
   /* skip start code */
   data += 4;
 
-  scr1 = GUINT32_FROM_BE (*(guint32 *) data);
-  scr2 = GUINT32_FROM_BE (*(guint32 *) (data + 4));
+  scr1 = GST_READ_UINT32_BE (data);
+  scr2 = GST_READ_UINT32_BE (data + 4);
 
   /* start parsing the stream */
   if ((*data & 0xc0) == 0x40) {
@@ -2260,7 +2235,7 @@ gst_flups_demux_scan_ts (GstFluPSDemux * demux, const guint8 * data,
        to DTS/PTS, that also implies 1 tick rounding error */
     data += 6;
     /* PMR:22 ! :2==11 ! reserved:5 ! stuffing_len:3 */
-    next32 = (GUINT32_FROM_BE ((*(guint32 *) data)));
+    next32 = GST_READ_UINT32_BE (data);
     if ((next32 & 0x00000300) != 0x00000300)
       goto beach;
 
