@@ -676,19 +676,17 @@ gst_scaletempo_set_property (GObject * object,
 static void
 gst_scaletempo_base_init (gpointer klass)
 {
-  static GstElementDetails element_details = {
-    "Scaletempo",
-    "Filter/Effect/Rate",
-    "Sync audio tempo with playback rate",
-    "Rov Juvano <rovjuvano@users.sourceforge.net>"
-  };
+
   GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
 
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&src_template));
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&sink_template));
-  gst_element_class_set_details (element_class, &element_details);
+  gst_element_class_set_details_simple (element_class, "Scaletempo",
+      "Filter/Effect/Rate",
+      "Sync audio tempo with playback rate",
+      "Rov Juvano <rovjuvano@users.sourceforge.net>");
 }
 
 static void
