@@ -45,15 +45,6 @@
 #include <gst/audio/gstaudiofilter.h>
 #include <gst/controller/gstcontroller.h>
 
-
-/* elementfactory information */
-static const GstElementDetails stereo_details =
-GST_ELEMENT_DETAILS ("Stereo effect",
-    "Filter/Effect/Audio",
-    "Muck with the stereo signal to enhance its 'stereo-ness'",
-    "Erik Walthinsen <omega@cse.ogi.edu>");
-
-
 #define ALLOWED_CAPS \
     "audio/x-raw-int,"                                                \
     " depth = (int) 16, "                                             \
@@ -95,7 +86,10 @@ gst_stereo_base_init (gpointer g_class)
   GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
   GstCaps *caps;
 
-  gst_element_class_set_details (element_class, &stereo_details);
+  gst_element_class_set_details_simple (element_class, "Stereo effect",
+      "Filter/Effect/Audio",
+      "Muck with the stereo signal to enhance its 'stereo-ness'",
+      "Erik Walthinsen <omega@cse.ogi.edu>");
 
   caps = gst_caps_from_string (ALLOWED_CAPS);
   gst_audio_filter_class_add_pad_templates (GST_AUDIO_FILTER_CLASS (g_class),
