@@ -310,23 +310,19 @@ HRESULT AudioFakeSink::CheckMediaType(const CMediaType *pmt)
 static void
 gst_dshowaudiodec_base_init (gpointer klass)
 {
-  GstDshowAudioDecClass *audiodec_class = (GstDshowAudioDecClass *)klass;
+  GstDshowAudioDecClass *audiodec_class = (GstDshowAudioDecClass *) klass;
   GstPadTemplate *src, *sink;
   GstCaps *srccaps, *sinkcaps;
   GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
-  GstElementDetails details;
+  char *description;
 
   audiodec_class->entry = tmp;
-  details.longname = g_strdup_printf ("DirectShow %s Decoder Wrapper",
-      tmp->element_longname);
-  details.klass = g_strdup ("Codec/Decoder/Audio");
-  details.description = g_strdup_printf ("DirectShow %s Decoder Wrapper",
-      tmp->element_longname);
-  details.author = "Sebastien Moutte <sebastien@moutte.net>";
-  gst_element_class_set_details (element_class, &details);
-  g_free (details.longname);
-  g_free (details.klass);
-  g_free (details.description);
+  description = g_strdup_printf ("DirectShow %s Decoder Wrapper",
+      tmp->element_description);
+  gst_element_class_set_details_simple (element_class, description,
+      "Codec/Decoder/Audio", description,
+      "Sebastien Moutte <sebastien@moutte.net>");
+  g_free (description);
 
   sinkcaps = gst_caps_from_string (tmp->sinkcaps);
 
@@ -358,13 +354,7 @@ gst_dshowaudiodec_class_init (GstDshowAudioDecClass * klass)
   gstelement_class->change_state =
       GST_DEBUG_FUNCPTR (gst_dshowaudiodec_change_state);
 
-  if (!parent_class)
-    parent_class = (GstElementClass *)g_type_class_ref (GST_TYPE_ELEMENT);
-
-  if (!dshowaudiodec_debug) {
-    GST_DEBUG_CATEGORY_INIT (dshowaudiodec_debug, "dshowaudiodec", 0,
-        "Directshow filter audio decoder");
-  }
+  parent_class = (GstElementClass *) g_type_class_peek_parent (klass);
 }
 
 static void
@@ -619,7 +609,9 @@ gst_dshowaudiodec_flush (GstDshowAudioDec * adec)
 
   /* flush dshow decoder and reset timestamp */
   adec->fakesrc->GetOutputPin()->Flush();
+
   adec->timestamp = GST_CLOCK_TIME_NONE;
+  adec->last_ret = GST_FLOW_OK;
 
   return TRUE;
 }
@@ -651,7 +643,7 @@ dshowaudiodec_set_input_format (GstDshowAudioDec *adec, GstCaps *caps)
    * decoder which doesn't need this */
   if (adec->layer == 1 || adec->layer == 2) {
     MPEG1WAVEFORMAT *mpeg1_format;
-    int version, samples;
+    int samples, version;
     GstStructure *structure = gst_caps_get_structure (caps, 0);
 
     size = sizeof (MPEG1WAVEFORMAT);
@@ -702,12 +694,37 @@ dshowaudiodec_set_input_format (GstDshowAudioDec *adec, GstCaps *caps)
   {
     size = sizeof (WAVEFORMATEX) +
         (adec->codec_data ? GST_BUFFER_SIZE (adec->codec_data) : 0);
-    format = (WAVEFORMATEX *)g_malloc0 (size);
-    if (adec->codec_data) {     /* Codec data is appended after our header */
-      memcpy (((guchar *) format) + sizeof (WAVEFORMATEX),
-          GST_BUFFER_DATA (adec->codec_data),
-          GST_BUFFER_SIZE (adec->codec_data));
-      format->cbSize = GST_BUFFER_SIZE (adec->codec_data);
+
+    if (adec->layer == 3) {
+      MPEGLAYER3WAVEFORMAT *mp3format;
+
+      /* The WinXP mp3 decoder doesn't actually check the size of this structure, 
+       * but requires that this be allocated and filled out (or we get obscure
+       * random crashes)
+       */
+      size = sizeof (MPEGLAYER3WAVEFORMAT);
+      mp3format = (MPEGLAYER3WAVEFORMAT *)g_malloc0 (size);
+      format = (WAVEFORMATEX *)mp3format;
+      format->cbSize = MPEGLAYER3_WFX_EXTRA_BYTES;
+
+      mp3format->wID = MPEGLAYER3_ID_MPEG;
+      mp3format->fdwFlags = MPEGLAYER3_FLAG_PADDING_ISO; /* No idea what this means for a decoder */
+
+      /* The XP decoder divides by nBlockSize, so we must set this to a
+         non-zero value, but it doesn't matter what - this is meaningless
+         for VBR mp3 anyway */
+      mp3format->nBlockSize = 1;
+      mp3format->nFramesPerBlock = 1;
+      mp3format->nCodecDelay = 0;
+    }
+    else {
+      format = (WAVEFORMATEX *)g_malloc0 (size);
+      if (adec->codec_data) {     /* Codec data is appended after our header */
+        memcpy (((guchar *) format) + sizeof (WAVEFORMATEX),
+            GST_BUFFER_DATA (adec->codec_data),
+            GST_BUFFER_SIZE (adec->codec_data));
+        format->cbSize = GST_BUFFER_SIZE (adec->codec_data);
+      }
     }
 
     format->wFormatTag = codec_entry->format;
@@ -1090,7 +1107,7 @@ dshow_adec_register (GstPlugin * plugin)
       type = g_type_register_static (GST_TYPE_ELEMENT,
           audio_dec_codecs[i].element_name, &info, (GTypeFlags)0);
       if (!gst_element_register (plugin, audio_dec_codecs[i].element_name,
-              GST_RANK_PRIMARY, type)) {
+              GST_RANK_SECONDARY, type)) {
         return FALSE;
       }
       GST_CAT_DEBUG (dshowaudiodec_debug, "Registered %s",
