@@ -33,11 +33,6 @@
 GST_DEBUG_CATEGORY_STATIC (dvb_base_bin_debug);
 #define GST_CAT_DEFAULT dvb_base_bin_debug
 
-static GstElementDetails dvb_base_bin_details = GST_ELEMENT_DETAILS ("DVB bin",
-    "Source/Bin/Video",
-    "Access descramble and split DVB streams",
-    "Alessandro Decina <alessandro@nnva.org>");
-
 static GstStaticPadTemplate src_template =
 GST_STATIC_PAD_TEMPLATE ("src%d", GST_PAD_SRC,
     GST_PAD_REQUEST,
@@ -208,7 +203,10 @@ dvb_base_bin_base_init (gpointer klass)
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&src_template));
 
-  gst_element_class_set_details (element_class, &dvb_base_bin_details);
+  gst_element_class_set_details_simple (element_class, "DVB bin",
+      "Source/Bin/Video",
+      "Access descramble and split DVB streams",
+      "Alessandro Decina <alessandro@nnva.org>");
 }
 
 static void
@@ -975,7 +973,7 @@ dvb_base_bin_uri_get_type (void)
 static gchar **
 dvb_base_bin_uri_get_protocols (void)
 {
-  static gchar *protocols[] = { "dvb", NULL };
+  static gchar *protocols[] = { (char *) "dvb", NULL };
 
   return protocols;
 }
